# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import random
import time
import unittest
from lazygal import newsize


class TestSizeParser(unittest.TestCase):

    def setUp(self):
        random.seed(time.time())

    def get_random_dimint(self):
        return random.randint(300, 2000)

    def get_random_size(self):
        return (self.get_random_dimint(), self.get_random_dimint())

    def get_random_percentage(self):
        return random.randint(10, 200)

    def assert_ratio_matches(self, orig_size, new_size):
        forig_size = list(map(float, orig_size))
        fnew_size = list(map(float, new_size))
        orig_ratio = forig_size[1] // forig_size[0]
        new_ratio = fnew_size[1] // fnew_size[0]
        self.assertAlmostEqual(orig_ratio, new_ratio, 2)

    def check(self, orig_size, size_string, expected_new_size, ratiocheck=True):
        newsizer = newsize.get_newsizer(size_string)
        dest_size = newsizer.dest_size(orig_size)
        self.assertEqual(expected_new_size, dest_size)
        if ratiocheck:
            self.assert_ratio_matches(orig_size, dest_size)

    def test_scale(self):
        orig_size = self.get_random_size()
        scale = self.get_random_percentage()
        size_string = "%d%%" % scale
        real_dest_size = tuple(map(lambda x: x * scale // 100, orig_size))

        self.check(orig_size, size_string, real_dest_size)

        orig_size = (1600, 1200)
        size_string = "50%"
        real_dest_size = (800, 600)

        self.check(orig_size, size_string, real_dest_size)

    def test_xyscale(self):
        orig_size = self.get_random_size()
        xscale = self.get_random_percentage()
        yscale = self.get_random_percentage()
        size_string = "%d%%%d%%" % (xscale, yscale)
        x = orig_size[0]
        y = orig_size[1]
        real_dest_size = (x * xscale // 100, y * yscale // 100)

        self.check(orig_size, size_string, real_dest_size, ratiocheck=False)

        orig_size = (1600, 1200)
        size_string = "50%75%"
        real_dest_size = (800, 900)

        self.check(orig_size, size_string, real_dest_size, ratiocheck=False)

    def test_width(self):
        orig_size = (1024, 768)
        size_string = "640"
        real_dest_size = (640, 480)

        self.check(orig_size, size_string, real_dest_size)

    def test_height(self):
        orig_size = (1024, 768)
        size_string = "x1200"
        real_dest_size = (1600, 1200)

        self.check(orig_size, size_string, real_dest_size)

    def test_maxwidthheight(self):
        self.check((1024, 768), "800x600", (800, 600))
        self.check((768, 1024), "800x600", (450, 600))

    def test_minwidthheight(self):
        self.check((1024, 768), "800x600^", (800, 600))
        self.check((768, 1024), "800x600^", (800, 1066))

    def test_mandatorywidthheight(self):
        self.check((1024, 768), "800x600!", (800, 600), ratiocheck=False)
        self.check((768, 1024), "800x600!", (800, 600), ratiocheck=False)

    def test_widthheightiflarger(self):
        self.check((1024, 768), "800x600>", (800, 600))
        self.check((768, 1024), "800x600>", (450, 600))
        self.check((420, 200), "800x600>", (420, 200))

    def test_widthheightifsmaller(self):
        self.check((1024, 768), "800x600<", (1024, 768))
        self.check((768, 1024), "800x600<", (768, 1024))
        self.check((420, 200), "800x600<", (1260, 600))
        self.check((420, 700), "800x600<", (420, 700))

    def test_area(self):
        self.check((1024, 768), "480000@", (800, 600))
        self.check((768, 1024), "480000@", (600, 800))


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2023 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import gettext
import locale
import logging
import os
import sys
from optparse import OptionParser


# i18n
from . import INSTALL_MODE, INSTALL_PREFIX

if INSTALL_MODE == "source":
    LOCALES_PATH = os.path.normpath(
        os.path.join(os.path.dirname(__file__), "..", "build", "mo")
    )
elif INSTALL_MODE == "installed":
    LOCALES_PATH = os.path.join(INSTALL_PREFIX, "share", "locale")
gettext.install("lazygal", LOCALES_PATH)

locale.setlocale(locale.LC_ALL, "")


from . import __version__
from . import config
from . import generators
from . import config
from . import eyecandy
from . import log


def main():
    usage = _("usage: %prog [options] albumdir")
    parser = OptionParser(usage=usage)

    # The help option must be changed to comply with i18n.
    parser.get_option("-h").help = _("Show this help message and exit.")

    parser.add_option(
        "",
        "--quiet",
        action="store_true",
        dest="quiet",
        help=_("Don't output anything except for errors."),
    )
    parser.add_option(
        "",
        "--debug",
        action="store_true",
        dest="debug",
        help=_("Output everything that lazygal is doing."),
    )
    parser.add_option(
        "-o",
        "--output-directory",
        action="store",
        type="string",
        dest="dest_dir",
        help=_(
            "Directory where web pages, slides and thumbs will be written (default is current directory)."
        ),
    )
    parser.add_option(
        "-t",
        "--theme",
        action="store",
        type="string",
        dest="theme",
        help=_("Theme name (looked up in theme directory) or theme full path."),
    )
    parser.add_option(
        "",
        "--default-style",
        action="store",
        type="string",
        dest="default_style",
        help=_("Default style to apply to the theme."),
    )
    parser.add_option(
        "",
        "--template-vars",
        action="store",
        type="string",
        dest="tpl_vars",
        help=_("Common variables to load all templates with."),
    )
    parser.add_option(
        "-f",
        "--force-gen-pages",
        action="store_true",
        dest="force_gen_pages",
        help=_("Force rebuild of all pages."),
    )
    parser.add_option(
        "",
        "--clean-destination",
        action="store_true",
        dest="clean_destination",
        help=_("Clean destination directory of files that should not be there."),
    )
    parser.add_option(
        "",
        "--preserve",
        type="string",
        action="append",
        metavar=_("PATTERN"),
        dest="preserve",
        help=_("Specifies pathname(s) which will be ignored during final cleanup"),
    )
    parser.add_option(
        "-v",
        "--version",
        action="store_true",
        dest="show_version",
        help=_("Display program version."),
    )
    parser.add_option(
        "",
        "--check-all-dirs",
        action="store_true",
        dest="check_all_dirs",
        help=_(
            "Exhaustively go through all directories regardless of source modification time."
        ),
    )
    parser.add_option(
        "",
        "--dir-flattening-depth",
        action="store",
        type="int",
        dest="dir_flattening_depth",
        help=_(
            "Level below which the directory tree is flattened. Default is 'No' which disables this feature."
        ),
    )
    parser.add_option(
        "-s",
        "--image-size",
        action="store",
        type="string",
        dest="image_size",
        help=_(
            "Size of images, define as <name>=SIZE,..., eg. small=800x600,medium=1024x768. The special value 0x0 uses original size. See manual page for SIZE syntax."
        ),
    )
    parser.add_option(
        "-T",
        "--thumbnail-size",
        action="store",
        type="string",
        dest="thumbnail_size",
        help=_(
            "Size of thumbnails, define as SIZE, eg. 150x113. See manual page for SIZE syntax."
        ),
    )
    parser.add_option(
        "-q",
        "--quality",
        action="store",
        type="int",
        dest="quality",
        help=_("Quality of generated JPEG images (default is 85)."),
    )
    parser.add_option(
        "-O",
        "--original",
        action="store_true",
        dest="original",
        help=_("Include original photos in output."),
    )
    parser.add_option(
        "",
        "--orig-base",
        action="store",
        type="string",
        dest="orig_base",
        help=_(
            "Do not copy original photos in output directory, instead link them using submitted relative path as base."
        ),
    )
    parser.add_option(
        "",
        "--orig-symlink",
        action="store_true",
        dest="orig_symlink",
        help=_(
            "Do not copy original photos in output directory, instead create symlinks to their original locations."
        ),
    )
    parser.add_option(
        "",
        "--puburl",
        action="store",
        type="string",
        dest="puburl",
        help=_("Publication URL (only useful for feed generation)."),
    )
    parser.add_option(
        "-m",
        "--generate-metadata",
        action="store_true",
        dest="metadata",
        help=_(
            "Generate metadata description files where they don't exist instead of generating the web gallery."
        ),
    )
    parser.add_option(
        "-n",
        "--thumbs-per-page",
        action="store",
        type="int",
        dest="thumbs_per_page",
        help=_(
            "Maximum number of thumbs per index page. This enables index pagination (0 is unlimited)."
        ),
    )
    parser.add_option(
        "-z",
        "--make-dir-zip",
        action="store_true",
        dest="dirzip",
        help=_("Make a zip archive of original pictures for each directory."),
    )
    parser.add_option(
        "",
        "--webalbum-pic-bg",
        action="store",
        type="string",
        dest="webalbumpic_bg",
        help=_(
            "Webalbum picture background color. Default is transparent, and implies the PNG format. Any other value, e.g. red, white, blue, uses JPEG."
        ),
    )
    parser.add_option(
        "",
        "--webalbum-pic-type",
        action="store",
        type="choice",
        choices=list(eyecandy.WEBALBUMPIC_TYPES.keys()),
        dest="webalbumpic_type",
        help=_("Webalbum picture type. Default is messy."),
    )
    parser.add_option(
        "",
        "--pic-sort-by",
        action="store",
        metavar=_("ORDER"),
        dest="pic_sort_by",
        help=_(
            "Sort order for images in a folder: filename, numeric, mtime, or exif. Add ':reverse' to reverse the chosen order."
        ),
    )
    parser.add_option(
        "",
        "--subgal-sort-by",
        action="store",
        metavar=_("ORDER"),
        dest="subgal_sort_by",
        help=_(
            "Sort order for sub galleries in a folder: dirname, numeric, exif or mtime. Add ':reverse' to reverse the chosen order."
        ),
    )
    parser.add_option(
        "",
        "--filter-by-tag",
        type="string",
        action="append",
        metavar=_("TAG"),
        dest="filter_by_tag",
        help=_(
            "Only include in the gallery pics whose IPTC keywords match the supplied filter(s)."
        ),
    )
    parser.add_option(
        "",
        "--exclude",
        type="string",
        action="append",
        metavar=_("PATTERN"),
        dest="exclude",
        help=_(
            "Regular expression pattern(s) describing directories or filenames to exclude from consideration."
        ),
    )
    parser.add_option(
        "",
        "--keep-gps-data",
        action="store_true",
        dest="keep_gps",
        help=_(
            "Do not remove GPS location tags from EXIF metadata. Mostly useful with holiday photos."
        ),
    )
    parser.add_option(
        "",
        "--no-video",
        action="store_true",
        dest="novideo",
        help=_("Do not process videos nor include them in indexes."),
    )
    (options, args) = parser.parse_args()

    if options.show_version:
        print(_("lazygal version %s") % __version__)
        sys.exit(0)

    if len(args) != 1:
        parser.print_help()
        sys.exit(_("Bad command line: wrong number of arguments."))

    source_dir = args[0]
    if not os.path.isdir(source_dir):
        print(_("Directory %s does not exist.") % source_dir)
        sys.exit(1)

    cmdline_config = config.LazygalConfig()
    for section in cmdline_config.valid_sections:
        cmdline_config.add_section(section)

    if options.quiet:
        cmdline_config.set("runtime", "quiet", True)
    if options.debug:
        cmdline_config.set("runtime", "debug", True)
    if options.check_all_dirs:
        cmdline_config.set("runtime", "check-all-dirs", True)

    if options.dest_dir is not None:
        cmdline_config.set("global", "output-directory", options.dest_dir)
    if options.force_gen_pages:
        cmdline_config.set("global", "force-gen-pages", True)
    if options.clean_destination:
        cmdline_config.set("global", "clean-destination", True)
    if options.preserve is not None:
        cmdline_config.set("global", "preserve_args", options.preserve)
    if options.dir_flattening_depth is not None:
        cmdline_config.set(
            "global", "dir-flattening-depth", options.dir_flattening_depth
        )
    if options.puburl is not None:
        cmdline_config.set("global", "puburl", options.puburl)
    if options.theme is not None:
        cmdline_config.set("global", "theme", options.theme)
    if options.exclude is not None:
        cmdline_config.set("global", "exclude_args", options.exclude)

    if options.default_style is not None:
        cmdline_config.set("webgal", "default-style", options.default_style)
    if options.webalbumpic_bg is not None:
        cmdline_config.set("webgal", "webalbumpic-bg", options.webalbumpic_bg)
    if options.webalbumpic_type is not None:
        cmdline_config.set("webgal", "webalbumpic-type", options.webalbumpic_type)
    if options.image_size is not None:
        cmdline_config.set("webgal", "image-size", options.image_size)
    if options.thumbnail_size is not None:
        cmdline_config.set("webgal", "thumbnail-size", options.thumbnail_size)
    if options.thumbs_per_page is not None:
        cmdline_config.set("webgal", "thumbs-per-page", options.thumbs_per_page)
    if options.pic_sort_by is not None:
        cmdline_config.set("webgal", "sort-medias", options.pic_sort_by)
    if options.subgal_sort_by is not None:
        cmdline_config.set("webgal", "sort-subgals", options.subgal_sort_by)
    if options.filter_by_tag is not None:
        cmdline_config.set("webgal", "filter-by-tag", options.filter_by_tag)
    if options.original:
        cmdline_config.set("webgal", "original", "Yes")
    if options.orig_base is not None:
        cmdline_config.set("webgal", "original-baseurl", options.orig_base)
    if options.orig_symlink:
        try:
            os.symlink
        except AttributeError:
            print(_("Option --orig-symlink is not available on this platform."))
            sys.exit(1)
        else:
            cmdline_config.set("webgal", "original-symlink", True)
    if options.dirzip:
        cmdline_config.set("webgal", "dirzip", True)
    if options.quality is not None:
        cmdline_config.set("webgal", "jpeg-quality", options.quality)
    if options.keep_gps:
        cmdline_config.set("webgal", "keep-gps", True)
    if options.novideo:
        cmdline_config.set("webgal", "novideo", True)

    if options.tpl_vars is not None:
        cmdline_config.add_section("template-vars")
        tpl_vars_defs = options.tpl_vars.split(",")
        for single_def in tpl_vars_defs:
            name, value = single_def.split("=")
            cmdline_config.set("template-vars", name, value)

    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    if sys.stdout.isatty():
        logging_handler = log.ProgressConsoleHandler
    else:
        logging_handler = logging.StreamHandler
    output_log = logging_handler(sys.stdout)
    output_log.setFormatter(logging.Formatter("%(message)s"))
    logger.addHandler(output_log)

    try:
        album = generators.Album(source_dir, cmdline_config)
    except ValueError as e:
        print(e)
        sys.exit(1)
    else:
        if sys.stdout.isatty():
            progress = generators.AlbumGenProgress(
                len(album.stats()["bydir"].keys()), album.stats()["total"]
            )

            def update_progress():
                output_log.update_progress(str(progress))

            progress.updated = update_progress
            progress.updated()
        else:
            progress = None

    if options.metadata:
        album.generate_default_metadata()
    else:
        try:
            album.generate(progress=progress)
        except KeyboardInterrupt:
            print(_("Interrupted."), file=sys.stderr)
            sys.exit(1)


# vim: ts=4 sw=4 expandtab



#!/usr/bin/env python

import sys

from lazygal.metadata import GExiv2

fn = sys.argv[1]
comment = sys.argv[2]

im = GExiv2.Metadata(fn)

# Assume comment is in utf-8, more encoding processing example using
# sys.stdin.encoding and example processing in lazygal/metadata.py
im["Exif.Photo.UserComment"] = comment
im.save_file()



#!/usr/bin/env python3
#
# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2022 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


# Convenience script to launch lazygal from source


import lazygal.cmdline


lazygal.cmdline.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import time
import shutil
import logging


class CircularDependency(Exception):
    pass


class MakeTask(object):
    """
    A simple task that remembers the last time it was built.
    """

    def __init__(self):
        self.deps = []
        self.output_items = []
        self.stamp_delete()
        self.__dep_only = False
        self._deps_populated = False
        self.update_build_status()

    def call_populate_deps(self):
        if not self._deps_populated:
            self.populate_deps()
            self._deps_populated = True

    def populate_deps(self):
        """
        Sometimes, deps should not be populated at init time, but later.
        This function is used to do this.
        """
        assert not self._deps_populated

    def add_dependency(self, dependency):
        if self in dependency.deps:
            raise CircularDependency("%s <-> %s" % (self, dependency))
        if dependency in self.deps:
            raise RuntimeError("adding same dep twice")
        else:
            self.deps.append(dependency)
            for output_item in dependency.output_items:
                self.register_output(output_item)

    def add_file_dependency(self, file_path):
        self.add_dependency(FileSimpleDependency(file_path))

    def get_mtime(self):
        return self.__last_build_time

    def set_dep_only(self):
        """
        Set this task to being only used as an intermediate work, its output
        is not required in the end. It won't be taken into account when
        computing whether a depending task should be built, even if it is
        older. But make() will be called if the depending task is built.
        """
        self.__dep_only = True

    def is_dep_only(self):
        return self.__dep_only

    def stamp_build(self, build_time=None):
        if not build_time:
            build_time = time.time()
        self.__last_build_time = build_time
        self.__built_once = True

    def stamp_delete(self):
        self.__last_build_time = -1  # older than oldest epoch
        self.__built_once = False

    def built_once(self):
        return self.__built_once

    def update_build_status(self):
        """
        Do stuff to update the build status (e.g. probe filesystem, deps).
        """
        pass

    def needs_build(self):
        self.call_populate_deps()

        if not self.built_once():
            logging.debug("%s build needed: never built", self)
            return True

        for dependency in self.deps:
            if not dependency.is_dep_only():
                mtime_gap = dependency.get_mtime() - self.get_mtime()
                if mtime_gap > 0 or dependency.needs_build():
                    logging.debug(
                        "%s build needed: dep %s newer by %ss",
                        self,
                        dependency,
                        mtime_gap,
                    )
                    return True
                else:
                    logging.debug(
                        "%s build: dep %s older by %ss", self, dependency, mtime_gap
                    )
        return False

    def make(self, force=False):
        self.call_populate_deps()
        if force or self.needs_build():
            for d in self.deps:
                d.make()  # dependency building not forced
            self.call_build()

    def call_build(self):
        """
        This method is really simple in this implementation, but it can be
        overridden with more complicated things in subclasses. The purpose is
        to setup some state before and/or after build.
        """
        try:
            self.build()
        except KeyboardInterrupt:
            self.clean_output()
            raise
        self.stamp_build()

    def build(self):
        """
        This method should be implemented in subclasses to define what the
        task should do.
        """
        raise NotImplementedError

    def register_output(self, output):
        """
        This provides a facility to register within the makefile machinery what
        items are built from the task.
        """
        self.output_items.append(output)

    def clean_output(self):
        """
        Clean-up in case of interruption (KeyboardInterrupt).
        """
        pass

    def print_dep_entry(self, level=0):
        indent = ""
        for index in range(0, level):
            indent = indent + "\t"

        print(indent, self, self.get_mtime())

    def print_dep_tree(self, depth=1, parent_level=-1):
        level = parent_level + 1
        if level > depth:
            return

        self.print_dep_entry(level)

        self.call_populate_deps()
        for d in self.deps:
            d.print_dep_tree(depth, level)


class GroupTask(MakeTask):
    """
    A class that builds nothing but groups subtasks.
    """

    def built_once(self):
        return True  # GroupTask is all about the deps.

    def update_build_status(self):
        super().update_build_status()
        # Find youngest dep, which should indicate latest build.
        mtime = None
        for dependency in self.deps:
            if dependency.built_once():
                new_mtime = dependency.get_mtime()
                if mtime is None or new_mtime > mtime:
                    mtime = new_mtime
        if mtime is None:
            self.stamp_delete()
        else:
            self.stamp_build(mtime)

    def add_dependency(self, dependency):
        super().add_dependency(dependency)
        dep_mtime = dependency.get_mtime()
        if dep_mtime > self.get_mtime():
            self.stamp_build(dep_mtime)

    def build(self):
        pass


class FileMakeObject(MakeTask):
    """
    A generic task to build a file.
    """

    def __init__(self, path):
        self._path = path
        super().__init__()
        self.register_output(self._path)

    def update_build_status(self):
        super().update_build_status()
        # Update build info according to file existence
        if os.path.exists(self._path):
            self.stamp_build(os.path.getmtime(self._path))
        else:
            self.stamp_delete()

    def clean_output(self):
        if os.path.lexists(self._path):
            os.unlink(self._path)

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self._path.encode("utf-8"))


class FileSimpleDependency(FileMakeObject):
    """
    Simple file dependency that needn't build. It just should be there.
    """

    def __init__(self, path):
        super().__init__(path)
        assert self.built_once(), path

    def build(self):
        pass

    def clean_output(self):
        pass


class FileCopy(FileMakeObject):
    """
    Simple file copy make target.
    """

    def __init__(self, src, dst):
        self.src = src
        self.path = dst
        super().__init__(dst)
        self.add_file_dependency(self.src)

    def build(self):
        if os.path.islink(self.path):
            os.remove(self.path)
        shutil.copyfile(self.src, self.path)


class FileSymlink(FileMakeObject):
    """
    Simple file symlink make target.
    """

    def __init__(self, src, dst):
        self.src = src
        self.dst = dst
        super().__init__(dst)
        self.add_file_dependency(self.src)

    def build(self):
        if os.path.islink(self.dst):
            os.remove(self.dst)
        os.symlink(self.src, self.dst)


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import logging

from PIL import Image as PILImage

# lazygal has her own ImageFile class, so avoid trouble
from PIL import ImageFile as PILImageFile

PILImageFile.MAXBLOCK = 1024 * 1024  # default is 64k, not enough for big pics

from . import make
from . import genfile
from . import eyecandy
from . import mediautils
from .metadata import GExiv2


THUMB_SIZE_NAME = "thumb"
VIDEO_SIZE_NAME = "video"


class ResizedMedia(genfile.WebalbumFile):

    force_extension = None

    def __init__(self, webgal, source_media, size_name):
        self.webgal = webgal
        self.source_media = source_media
        self.filename = self.webgal._add_size_qualifier(
            self.source_media.filename, size_name, self.force_extension
        )
        path = os.path.join(self.webgal.path, self.filename)
        super().__init__(path, webgal)

        self.newsizer = self.webgal.newsizers[size_name]
        self.size = None

        self.add_dependency(self.source_media)

    def get_size(self):
        if self.size is None:
            self.size = self.newsizer.dest_size(self.source_media.get_size())
        return self.size

    def get_verb(self):
        raise NotImplementedError

    VERB = property(get_verb)

    def build(self):
        media_rel_path = self.rel_path(self.webgal.flattening_dir)
        logging.info("  %s %s", self.VERB, media_rel_path)
        logging.debug("(%s)", self.path)

        if not self.source_media.broken:
            self.do_build()


class ImageOtherSize(ResizedMedia):

    def __init__(self, webgal, source_image, size_name):
        if "alphachannel" in source_image.md and source_image.md["alphachannel"]:
            self.force_extension = ".png"
        else:
            self.force_extension = ".jpg"
        super().__init__(webgal, source_image, size_name)

        self.rotation = None

    def get_verb(self):
        return _("RESIZE")

    VERB = property(get_verb)

    def resize(self, im):
        new_size = self.get_size()

        im.draft(None, new_size)
        return im.resize(new_size, PILImage.LANCZOS)

    def save(self, im):
        if (
            "alphachannel" in self.source_media.md
            and self.source_media.md["alphachannel"]
        ):
            self.save_png(im)
        else:
            self.save_jpeg(im)

    def save_png(self, im):
        with open(self.path, "w+b") as im_fp:
            im.save(
                im_fp, "png", quality=self.webgal.quality, **self.webgal.save_options
            )

    def save_jpeg(self, im):
        calibrated = False
        while not calibrated:
            with open(self.path, "w+b") as im_fp:
                try:
                    if im.mode != "RGB":
                        # convert indexed images into RGB mode, usefull
                        # for PNG indexed images
                        im = im.convert("RGB")
                    im.save(
                        im_fp,
                        "jpeg",
                        quality=self.webgal.quality,
                        **self.webgal.save_options
                    )
                except IOError as e:
                    if str(e).startswith("encoder error"):
                        PILImageFile.MAXBLOCK = 2 * PILImageFile.MAXBLOCK
                        continue
                    else:
                        raise
            calibrated = True

    def get_rotation(self):
        if self.rotation is None:
            if "rotation" in self.source_media.md["metadata"]:
                self.rotation = self.source_media.md["metadata"]["rotation"]
            else:
                self.rotation = 0
        return self.rotation

    def get_size(self):
        if self.size is None:
            orig_size = self.source_media.get_size()
            if orig_size == (
                0,
                0,
            ):  # broken pic
                self.size = orig_size
            elif self.get_rotation() in (
                90,
                270,
            ):
                # swap coords
                orig_size = (
                    orig_size[1],
                    orig_size[0],
                )
                self.size = self.newsizer.dest_size(orig_size)
                self.unrotated_size = (
                    self.size[1],
                    self.size[0],
                )
            else:
                self.size = self.newsizer.dest_size(orig_size)
                self.unrotated_size = self.size
        return self.size

    def get_image(self):
        with open(self.source_media.path, "rb") as im_fp:
            im = PILImage.open(im_fp)
            im.load()
            return im

    TRANSPOSE_METHODS = {
        90: PILImage.ROTATE_90,
        180: PILImage.ROTATE_180,
        270: PILImage.ROTATE_270,
    }

    def resize(self, im):
        rotation = self.get_rotation()
        self.get_size()

        im.draft(None, self.unrotated_size)
        im = im.resize(self.unrotated_size, PILImage.LANCZOS)

        # Use EXIF data to rotate target image if available and required
        if rotation != 0:
            im = im.transpose(self.TRANSPOSE_METHODS[rotation])

        return im

    PRIVATE_IMAGE_TAGS = (
        "Exif.GPSInfo.GPSLongitude",
        "Exif.GPSInfo.GPSLatitude",
        "Exif.GPSInfo.GPSDestLongitude",
        "Exif.GPSInfo.GPSDestLatitude",
    )

    def copy_metadata(self):
        imgtags = GExiv2.Metadata(self.source_media.path)
        dest_imgtags = GExiv2.Metadata(self.path)
        for tag in imgtags.get_exif_tags():
            try:
                dest_imgtags[tag] = imgtags[tag]
            except UnicodeDecodeError:
                logging.warning(_("Could not copy metadata tag '%s'"), tag)

        new_size = self.get_size()
        dest_imgtags["Exif.Photo.PixelXDimension"] = str(new_size[0])
        dest_imgtags["Exif.Photo.PixelYDimension"] = str(new_size[1])

        if self.get_rotation() != 0:
            # Smaller image has been rotated in order to be displayed correctly
            # in a web browser. Fix orientation tag accordingly.
            dest_imgtags["Exif.Image.Orientation"] = "1"

        # Those are removed from published pics due to pivacy concerns,
        # unless explicitly told to keep them in. Option to retain GPS
        # tags can only be set from command line.
        if not self.webgal.keep_gps:
            for tag in self.PRIVATE_IMAGE_TAGS:
                try:
                    dest_imgtags.try_clear_tag(tag)
                except KeyError:
                    pass
        try:
            dest_imgtags.save_file()
        except Exception as e:
            logging.error(_("Could not copy metadata in reduced picture: %s"), e)

    def do_build(self):
        try:
            im = self.resize(self.get_image())
        except OSError:
            self.source_media.set_broken()
            # Make the system believe the file was built a long time ago.
            self.stamp_build(0)
            self.clean_output()
        else:
            self.save(im)
            if self.webgal.config.get("webgal", "publish-metadata"):
                self.copy_metadata()


class VideoThumb(ResizedMedia):

    force_extension = ".jpg"

    def get_verb(self):
        return _("VIDEOTHUMB")

    VERB = property(get_verb)

    def do_build(self):
        try:
            mediautils.VideoThumbnailer(self.source_media.path).convert(
                self.path, self.get_size()
            )
        except mediautils.VideoError as e:
            logging.error(
                _("  creating %s thumbnail failed, skipped"), self.source_media.filename
            )
            logging.info(str(e))
            self.clean_output()


class WebalbumPicture(make.FileMakeObject):

    BASEFILENAME = "index"

    def __init__(self, webgal_dir):
        self.path = os.path.join(webgal_dir.path, webgal_dir.get_webalbumpic_filename())
        super().__init__(self.path)

        self.add_dependency(webgal_dir.source_dir)

        medias = [
            m for m in webgal_dir.source_dir.get_all_medias() if m.type == "image"
        ]

        # Add video thumbs
        for m in webgal_dir.medias:
            if m.media.type == "video":
                medias.append(m.thumb)

        for m in medias:
            self.add_dependency(m)

        if webgal_dir.source_dir.album_picture:
            albumpic_path = os.path.join(
                webgal_dir.source_dir.path, webgal_dir.source_dir.album_picture
            )
            if not os.path.isfile(albumpic_path):
                logging.error(
                    _("Supplied album picture %s does not exist."), albumpic_path
                )

            md_dirpic_thumb = webgal_dir._add_size_qualifier(
                webgal_dir.source_dir.album_picture, THUMB_SIZE_NAME
            )
            md_dirpic_thumb = os.path.join(webgal_dir.path, md_dirpic_thumb)
        else:
            md_dirpic_thumb = None

        pic_paths = [p.path for p in medias]

        multipic_repr = eyecandy.WEBALBUMPIC_TYPES[webgal_dir.webalbumpic_type]

        # Use 800x600 as a random value to obtain a 4:3 aspect ratio (if
        # thumb size preserves aspect ratio)
        self.dirpic = multipic_repr(
            pic_paths,
            md_dirpic_thumb,
            bg=webgal_dir.webalbumpic_bg,
            result_size=webgal_dir.webalbumpic_size,
        )

    def build(self):
        logging.info(_("  DIRPIC %s"), os.path.basename(self.path))
        logging.debug("(%s)", self.path)
        try:
            self.dirpic.write(self.path)
        except ValueError as ex:
            logging.error(str(ex))


class WebVideo(genfile.WebalbumFile):

    def __init__(self, webgal, source_video, size_name, progress):
        self.progress = progress
        self.webgal = webgal
        self.source_video = source_video
        self.filename = self.webgal._add_size_qualifier(
            self.source_video.filename, size_name, ".webm"
        )
        path = os.path.join(self.webgal.path, self.filename)
        super().__init__(path, webgal)

        newsizer = self.webgal.newsizers[size_name]
        if newsizer == "original":
            self.new_width, self.new_height = (None, None)
        else:
            self.new_width, self.new_height = newsizer.dest_size(
                self.source_video.get_size()
            )

        self.add_dependency(self.source_video)

    def build(self):
        vid_rel_path = self.rel_path(self.webgal.flattening_dir)
        logging.info(_("  TRANSCODE %s"), vid_rel_path)

        try:
            transcoder = mediautils.WebMTranscoder(self.source_video.path)
            if self.new_width is not None and self.new_height is not None:
                transcoder.scale((self.new_width, self.new_height))
            transcoder.set_progress(self.progress)
            transcoder.convert(self.path)
        except mediautils.VideoError as e:
            logging.error(
                _("  transcoding %s failed, skipped"), self.source_video.filename
            )
            logging.info(str(e))
            self.source_video.set_broken()
            self.clean_output()
        finally:
            self.progress.set_task_done()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import locale
import logging
import gc
import genshi
import sys
import re
import fnmatch
import shutil

from .config import LazygalConfig
from .config import USER_CONFIG_PATH, LazygalConfigDeprecated
from .sourcetree import SOURCEDIR_CONFIGFILE
from .metadata import GExiv2

from . import make
from . import pathutils
from . import sourcetree
from . import pindex
from . import tpl
from . import newsize
from . import metadata
from . import genpage
from . import genmedia
from . import genfile
from . import mediautils
from . import theme


from lazygal import INSTALL_MODE, INSTALL_PREFIX

if INSTALL_MODE == "source":
    DATAPATH = os.path.normpath(os.path.join(os.path.dirname(__file__), ".."))
elif INSTALL_MODE == "installed":
    DATAPATH = os.path.join(INSTALL_PREFIX, "share", "lazygal")
    if not os.path.exists(os.path.join(DATAPATH, "themes")):
        print(_("Could not find themes dir, check your installation!"))
        sys.exit(1)


DEST_SHARED_DIRECTORY_NAME = "shared"


class SubgalSort(make.MakeTask):
    """
    This task sorts the medias within a gallery according to the chosen rule.
    """

    def __init__(self, webgal_dir):
        super().__init__()
        self.set_dep_only()
        self.webgal_dir = webgal_dir

    def build(self):
        logging.info(_("  SORTING pics and subdirs"))

        order = self.webgal_dir.subgal_sort_by["order"]
        if order == "exif":
            subgal_sortkey = lambda x: x.source_dir.latest_media_stamp()
        elif order == "mtime":
            subgal_sortkey = lambda x: x.source_dir.get_mtime()
        elif order == "numeric":
            subgal_sortkey = lambda x: x.source_dir.name_numeric()
        elif order == "dirname" or order == "filename":  # Backward compat
            subgal_sortkey = lambda x: x.source_dir.filename
        else:
            raise ValueError(
                _("Unknown sorting criterion '%s'") % self.webgal_dir.subgal_sort_by[0]
            )
        self.webgal_dir.subgals.sort(
            key=subgal_sortkey, reverse=self.webgal_dir.subgal_sort_by["reverse"]
        )

        order = self.webgal_dir.pic_sort_by["order"]
        if order == "exif":
            sortkey = lambda x: x.media.sortkey()
        elif order == "mtime":
            sortkey = lambda x: x.media.get_mtime()
        elif order == "numeric":
            sortkey = lambda x: x.media.name_numeric()
        elif order == "filename":
            sortkey = lambda x: x.media.filename
        else:
            raise ValueError(
                _("Unknown sorting criterion '%s'") % self.webgal_dir.pic_sort_by[0]
            )
        self.webgal_dir.medias.sort(
            key=sortkey, reverse=self.webgal_dir.pic_sort_by["reverse"]
        )

        # chain medias
        previous = None
        for media in self.webgal_dir.medias:
            if previous:
                previous.set_next(media)
                media.set_previous(previous)
            previous = media


class SubgalBreak(make.MakeTask):
    """
    This task breaks galleries into multiple pages.
    """

    def __init__(self, webgal_dir):
        super().__init__()
        self.webgal_dir = webgal_dir

        self.__last_page_number = -1

    def next_page_number(self):
        self.__last_page_number += 1
        return self.__last_page_number

    def how_many_pages(self):
        return self.__last_page_number + 1

    def build(self):
        logging.info(_("  BREAKING web gallery into multiple pages"))

        if self.webgal_dir.thumbs_per_page == 0:
            self.__fill_no_pagination()
        else:
            if self.webgal_dir.flatten_below():
                self.__fill_loose_pagination()
            else:
                self.__fill_real_pagination()

    def __fill_no_pagination(self):
        galleries = []
        galleries.append((self.webgal_dir, self.webgal_dir.medias))
        if self.webgal_dir.flatten_below():
            subgals = []
            for dir in self.webgal_dir.get_all_subgals():
                dir.call_populate_deps()
                galleries.append((dir, dir.medias))
        else:
            subgals = self.webgal_dir.subgals
        self.webgal_dir.add_index_page(subgals, galleries)

    def __fill_loose_pagination(self):
        """
        Loose pagination not breaking subgals (chosen if subgals are flattened).
        """
        subgals = []  # No subgal links as they are flattened

        galleries = []
        how_many_medias = 0
        subgals_it = iter([self.webgal_dir] + self.webgal_dir.get_all_subgals())
        for subgal in subgals_it:
            how_many_medias += subgal.pindex.get_media_count()
            galleries.append((subgal, subgal.medias))
            if how_many_medias > self.webgal_dir.thumbs_per_page:
                self.webgal_dir.add_index_page(subgals, galleries)
                galleries = []
                how_many_medias = 0

        if galleries:
            self.webgal_dir.add_index_page(subgals, galleries)

    def __fill_real_pagination(self):
        medias_amount = len(self.webgal_dir.medias)
        how_many_pages = medias_amount // self.webgal_dir.thumbs_per_page
        if medias_amount == 0 or medias_amount % self.webgal_dir.thumbs_per_page > 0:
            how_many_pages = how_many_pages + 1

        for page_number in range(0, how_many_pages):
            step = page_number * self.webgal_dir.thumbs_per_page
            end_index = step + self.webgal_dir.thumbs_per_page
            shown_medias = self.webgal_dir.medias[step:end_index]
            galleries = [(self.webgal_dir, shown_medias)]

            # subgal links only for first page
            if page_number == 0:
                subgals = self.webgal_dir.subgals
            else:
                subgals = []

            self.webgal_dir.add_index_page(subgals, galleries)


class WebalbumMediaTask(make.GroupTask):

    def __init__(self, webgal, media):
        super().__init__()

        self.webgal = webgal
        self.media = media

        self.previous = None
        self.next = None

        self.original = None
        self.resized = {}
        self.browse_pages = {}

        if self.webgal.original and not self.webgal.orig_base:
            self.add_dependency(self.get_original())

        for size_name in self.webgal.browse_sizes:
            self.resized[size_name] = self.get_resized(size_name)
            if self.resized[size_name] not in self.deps:
                self.add_dependency(self.resized[size_name])

            if self.webgal.album.theme.kind == "static":
                self.browse_pages[size_name] = self.get_browse_page(size_name)
                if self.webgal.album.force_gen_pages:
                    self.browse_pages[size_name].stamp_delete()

    def set_next(self, media):
        self.next = media
        if media:
            for bpage in self.browse_pages.values():
                if media.thumb:
                    bpage.add_dependency(media.thumb)

    def set_previous(self, media):
        self.previous = media
        if media:
            for bpage in self.browse_pages.values():
                if media.thumb:
                    bpage.add_dependency(media.thumb)

    def get_original_or_symlink(self):
        if not self.webgal.orig_symlink:
            return genfile.CopyMediaOriginal(self.webgal, self.media)
        else:
            return genfile.SymlinkMediaOriginal(self.webgal, self.media)

    def get_original(self):
        if not self.original:
            self.original = self.get_original_or_symlink()
        return self.original

    def get_browse_page(self, size_name):
        return genpage.WebalbumBrowsePage(self.webgal, size_name, self)

    def make(self):
        super().make()
        self.webgal.media_done()


class WebalbumImageTask(WebalbumMediaTask):
    """
    This task builds all items related to one picture.
    """

    def __init__(self, webgal, image):
        super().__init__(webgal, image)

        self.thumb = genmedia.ImageOtherSize(
            self.webgal, self.media, genmedia.THUMB_SIZE_NAME
        )
        self.add_dependency(self.thumb)

    def get_resized(self, size_name):
        if self.webgal.newsizers[size_name] == "original":
            return self.get_original_or_symlink()
        else:
            sized = genmedia.ImageOtherSize(self.webgal, self.media, size_name)
            if sized.get_size() == sized.source_media.get_size():
                # Do not process if size is the same
                return self.get_original()
            else:
                return sized


class WebalbumVideoTask(WebalbumMediaTask):
    """
    This task builds all items related to one video.
    """

    def __init__(self, webgal, video):
        self.webvideo = None

        super().__init__(webgal, video)

        self.thumb = genmedia.VideoThumb(
            self.webgal, self.media, genmedia.THUMB_SIZE_NAME
        )
        self.add_dependency(self.thumb)

    def get_resized(self, size_name):
        if not self.webvideo:
            self.webvideo = genmedia.WebVideo(
                self.webgal, self.media, genmedia.VIDEO_SIZE_NAME, self.webgal.progress
            )

        return self.webvideo


class WebalbumDir(make.GroupTask):
    """
    This is a built web gallery with its files, thumbs and reduced pics.
    """

    def __init__(self, dir, subgals, album, album_dest_dir, progress=None):
        self.source_dir = dir
        self.path = os.path.join(album_dest_dir, self.source_dir.strip_root())
        if self.path.endswith(os.sep):
            self.path = os.path.dirname(self.path)

        super().__init__()

        self.progress = progress

        self.subgals = subgals
        self.album = album
        self.feed = None

        self.flattening_dir = None

        self.config = LazygalConfig()
        self.config.load(self.album.config)
        self.__configure()

        self.pindex = pindex.PersistentIndex(self)
        self.add_dependency(self.pindex)
        self.webassets = pindex.WebAssets(self)
        self.webassets.add_dependency(self.pindex)

    def populate_deps(self):
        super().populate_deps()

        for s in self.subgals:
            if not s.has_media_below():
                self.subgals.remove(s)

        self.medias = []
        self.sort_task = SubgalSort(self)
        self.sort_task.add_dependency(self.source_dir)
        for media in self.source_dir.medias:
            media.load_metadata(self.pindex)
            if self.tagfilters:
                # tag-filtering is requested
                res = True
                for tagf in self.tagfilters:
                    # concatenate the list of tags as a string of words,
                    # space-separated.  to ensure that we match the full
                    # keyword and not only a subpart of it, we also surround
                    # the matching pattern with spaces

                    # we look for tag words, partial matches are not wanted
                    regex = re.compile(r"\b" + tagf + r"\b")

                    kwlist = " ".join(media.md["metadata"]["keywords"])
                    if re.search(regex, kwlist) is None:
                        res = False
                        break
                if not res:
                    self.pindex.unpublish_media(media)
                    self.media_done()
                    continue

            self.sort_task.add_dependency(media)

            media_task = None
            if media.type == "image":
                media_task = WebalbumImageTask(self, media)
            elif media.type == "video":
                if not self.config.get("webgal", "novideo"):
                    media_task = WebalbumVideoTask(self, media)
                else:
                    logging.warning(
                        _("Video support disabled: ignoring %s.") % media.filename
                    )
            else:
                raise NotImplementedError("Unknown media type '%s'" % media.type)

            if media_task:
                self.medias.append(media_task)
                self.add_dependency(media_task)

        self.index_pages = []

        if self.has_media_below() and not self.should_be_flattened():
            self.break_task = SubgalBreak(self)
            self.break_task.add_dependency(self.pindex)

            if self.thumbs_per_page > 0:
                self.break_task.add_dependency(self.sort_task)

            # This task is special because it populates dependencies. This is
            # why it needs to be built before a build check.
            self.break_task.make()

            self.webgal_pic = genmedia.WebalbumPicture(self)
            self.add_dependency(self.webgal_pic)
        else:
            self.break_task = None

        self.dirzip = None
        if self.config.get("webgal", "dirzip") and len(self.medias) > 1:
            self.dirzip = genfile.WebalbumArchive(self)
            self.add_dependency(self.dirzip)
            self.webassets.add_dependency(self.dirzip)

    def __parse_browse_sizes(self, sizes_defs):
        fixed_sizes_defs = sizes_defs
        if isinstance(sizes_defs, dict):
            # simpler defs
            fixed_sizes_defs = []
            first = True
            for name, defs in sizes_defs.items():
                fixed_sizes_defs.append(
                    {
                        "name": name,
                        "defs": defs,
                        "default": first,
                    }
                )
                first = False

        for size_defs in fixed_sizes_defs:
            name = size_defs["name"]
            if name == genmedia.THUMB_SIZE_NAME:
                raise ValueError(
                    _("Size name '%s' is reserved for internal processing.")
                    % genmedia.THUMB_SIZE_NAME
                )
            self.__parse_size(name, size_defs["defs"])
            self.browse_sizes.append(name)
            if "default" in size_defs and size_defs["default"]:
                self.default_size_name = name

    def __parse_size(self, size_name, size_string):
        if size_string == "0x0":
            self.newsizers[size_name] = "original"
        elif size_string in self.newsizers:
            pass  # size reference, do nothing
        else:
            try:
                self.newsizers[size_name] = newsize.get_newsizer(size_string)
            except newsize.NewsizeStringParseError:
                raise ValueError(
                    _("'%s' for size '%s' does not describe a known size syntax.")
                    % (
                        size_string,
                        size_name,
                    )
                )

    def __load_tpl_vars(self):
        # Load tpl vars from config
        tpl_vars = {}
        if self.config.has_section("template-vars"):
            tpl_vars = {}
            for option in self.config.options("template-vars"):
                value = self.config.get("template-vars", option)
                tpl_vars[option] = genshi.core.Markup(value)

        return tpl_vars

    def __configure(self):
        config_dirs = self.source_dir.parent_paths()[:-1]  # strip root dir
        config_dirs.reverse()  # from root to deepest
        config_files = list(
            map(lambda d: os.path.join(d, SOURCEDIR_CONFIGFILE), config_dirs)
        )
        logging.debug(
            _("  Trying loading gallery configs: %s"),
            ", ".join(map(self.source_dir.strip_root, config_files)),
        )
        for c in config_files:
            self.config.load_any(c)

        self.browse_sizes = []
        self.newsizers = {}
        self.__parse_browse_sizes(self.config.get("webgal", "image-size"))
        self.__parse_size(
            genmedia.THUMB_SIZE_NAME, self.config.get("webgal", "thumbnail-size")
        )
        self.__parse_size(
            genmedia.VIDEO_SIZE_NAME, self.config.get("webgal", "video-size")
        )

        self.tpl_vars = self.__load_tpl_vars()
        styles = self.album.theme.get_avail_styles(
            self.config.get("webgal", "default-style")
        )
        self.tpl_vars.update({"styles": styles})

        self.set_original(
            self.config.get("webgal", "original"),
            self.config.get("webgal", "original-baseurl"),
            self.config.get("webgal", "original-symlink"),
        )

        self.thumbs_per_page = self.config.get("webgal", "thumbs-per-page")

        self.quality = self.config.get("webgal", "jpeg-quality")
        self.save_options = {}
        if self.config.get("webgal", "jpeg-optimize"):
            self.save_options["optimize"] = True
        if self.config.get("webgal", "jpeg-progressive"):
            self.save_options["progressive"] = True

        self.pic_sort_by = self.config.get("webgal", "sort-medias")
        self.subgal_sort_by = self.config.get("webgal", "sort-subgals")
        self.tagfilters = self.config.get("webgal", "filter-by-tag")

        self.webalbumpic_bg = self.config.get("webgal", "webalbumpic-bg")
        self.webalbumpic_type = self.config.get("webgal", "webalbumpic-type")
        try:
            self.webalbumpic_size = list(
                map(int, self.config.get("webgal", "webalbumpic-size").split("x"))
            )
            if len(self.webalbumpic_size) != 2:
                raise ValueError
        except ValueError:
            logging.error(_("Bad syntax for webalbumpic-size."))
            sys.exit(1)
        self.keep_gps = self.config.get("webgal", "keep-gps")

    def set_original(self, original=False, orig_base=None, orig_symlink=False):
        self.original = original or orig_symlink
        self.orig_symlink = orig_symlink
        if self.original and orig_base and not orig_symlink:
            self.orig_base = orig_base
        else:
            self.orig_base = None

    def get_webalbumpic_filename(self):
        if self.webalbumpic_bg == "transparent":
            ext = ".png"  # JPEG does not have an alpha channel
        else:
            ext = ".jpg"
        return genmedia.WebalbumPicture.BASEFILENAME + ext

    def _add_size_qualifier(self, path, size_name, force_extension=None):
        filename, extension = os.path.splitext(path)
        if force_extension is not None:
            extension = force_extension

        if size_name == self.default_size_name and extension == ".html":
            # Do not append default size name to HTML page filename
            return path
        elif (
            size_name in self.browse_sizes
            and self.newsizers[size_name] == "original"
            and extension != ".html"
        ):
            # Do not append size_name to unresized images.
            return path
        else:
            return "%s_%s%s" % (filename, size_name, extension)

    def add_index_page(self, subgals, galleries):
        page_number = self.break_task.next_page_number()
        pages = []
        for size_name in self.browse_sizes:
            page = genpage.WebalbumIndexPage(
                self, size_name, page_number, subgals, galleries
            )
            if self.album.force_gen_pages:
                page.stamp_delete()
            self.add_dependency(page)
            pages.append(page)
        self.index_pages.append(pages)

    def register_output(self, output):
        # We only care about output in the current directory
        if os.path.dirname(output) == self.path:
            super().register_output(output)

    def register_feed(self, feed):
        self.feed = feed

    def get_subgal_count(self):
        if self.flatten_below():
            return 0
        else:
            return len(self.source_dir.subdirs)

    def get_all_subgals(self):
        all_subgals = list(self.subgals)  # We want a copy here.
        for subgal in self.subgals:
            all_subgals.extend(subgal.get_all_subgals())
        return all_subgals

    def has_media(self):
        if self.tagfilters:
            return True if self.medias else False
        else:
            return True if self.source_dir.medias else False

    def has_media_below(self):
        if self.has_media():
            return True
        for subgal in self.subgals:
            if subgal.has_media_below():
                return True
        return False

    def should_be_flattened(self, path=None):
        if path is None:
            path = self.source_dir.path
        return (
            self.album.dir_flattening_depth is not False
            and self.source_dir.get_album_level(path) > self.album.dir_flattening_depth
        )

    def flatten_below(self):
        if self.album.dir_flattening_depth is False:
            return False
        elif self.source_dir.subdirs:
            # As all subdirs are at the same level, if one should be flattened,
            # all should.
            return self.subgals[0].should_be_flattened()
        else:
            return False

    def rel_path_to_src(self, target_srcdir_path):
        """
        Returns the relative path to go from this directory to
        target_srcdir_path.
        """
        return self.source_dir.rel_path(self.source_dir.path, target_srcdir_path)

    def rel_path(self, path):
        """
        Returns the relative path to go from this directory to the path
        supplied as argument.
        """
        return os.path.relpath(path, self.path)

    def flattening_srcpath(self, srcdir_path):
        """
        Returns the source path in which srcdir_path should flattened, that is
        the path of the gallery index that will point to srcdir_path's
        pictures.
        """
        if self.should_be_flattened(srcdir_path):
            cur_path = srcdir_path
            while self.should_be_flattened(cur_path):
                cur_path, dummy = os.path.split(cur_path)
            return cur_path
        else:
            return ""

    def list_foreign_files(self):
        self.call_populate_deps()

        if not os.path.isdir(self.path):
            return []

        foreign_files = []

        # Check dest for junk files
        extra_files = []
        if self.source_dir.is_album_root():
            extra_files.append(os.path.join(self.path, DEST_SHARED_DIRECTORY_NAME))

        dirnames = [d.source_dir.name for d in self.subgals]
        expected_dirs = list(map(lambda dn: os.path.join(self.path, dn), dirnames))
        for dest_file in os.listdir(self.path):
            dest_file = os.path.join(self.path, dest_file)
            if (
                dest_file not in self.output_items
                and dest_file not in expected_dirs
                and dest_file not in extra_files
            ):
                foreign_files.append(dest_file)

        return foreign_files

    def skip_media(self, src_media):
        return False

    def needs_build_quick(self):
        """
        Faster check if needs to be built.
        """
        if self._deps_populated or self.tagfilters:
            return self.needs_build()

        return self.pindex.needs_build()

    def build(self):
        for dest_file in self.list_foreign_files():
            self.album.cleanup(dest_file, self.path)

    def make(self, force=False):
        super().make(force)
        self.update_build_status()

    def media_done(self):
        if self.progress is not None:
            self.progress.media_done()


class SharedFiles(make.FileMakeObject):

    def __init__(self, album, dest_dir, tpl_vars):
        self.path = os.path.join(dest_dir, DEST_SHARED_DIRECTORY_NAME)
        self.album = album

        # Create the shared files directory if it does not exist
        if not os.path.isdir(self.path):
            logging.info(_("MKDIR %SHAREDDIR%"))
            logging.debug("(%s)", self.path)
            os.makedirs(self.path)

        super().__init__(self.path)

        self.expected_shared_files = []
        for shared in self.album.theme.shared_files:
            shared_file_dest = os.path.join(self.path, shared["dest"])

            if self.album.theme.tpl_loader.is_known_template_type(shared["source"]):
                sf = genpage.SharedFileTemplate(
                    album, shared["source"], shared_file_dest, tpl_vars
                )
                if self.album.force_gen_pages:
                    sf.stamp_delete()
            else:
                sf = genfile.SharedFileCopy(shared["source"], shared_file_dest)
            self.expected_shared_files.append(sf.path)

            self.add_dependency(sf)

    def build(self):
        # Cleanup themes files which are not in themes anymore.
        for present_file in os.listdir(self.path):
            file_path = os.path.join(self.path, present_file)
            if file_path not in self.expected_shared_files:
                self.album.cleanup(file_path, self.path)


class DummyProgress(object):

    def dir_done(self):
        pass

    def media_done(self, how_many=1):
        pass

    def set_task_progress(self, percent):
        pass

    def set_task_done(self):
        pass

    def updated(self):
        pass


class AlbumGenProgress(object):

    def __init__(self, dirs_total, medias_total):
        self._dirs_total = dirs_total
        self._dirs_done = 0

        self._medias_total = medias_total
        self._medias_done = 0

        self._task_percent = None

    def dir_done(self):
        self._dirs_done = self._dirs_done + 1
        self.updated()

    def media_done(self, how_many=1):
        self._medias_done = self._medias_done + how_many
        self.updated()

    def set_task_progress(self, percent):
        self._task_percent = percent
        self.updated()

    def set_task_done(self):
        self._task_percent = None
        self.updated()

    def __str__(self):
        msg = []

        if self._dirs_total > 0:
            msg.append(
                "dir %d/%d (%d%%)"
                % (
                    self._dirs_done,
                    self._dirs_total,
                    100 * self._dirs_done // self._dirs_total,
                )
            )

        if self._medias_total > 0:
            msg.append(
                "media %d/%d (%d%%)"
                % (
                    self._medias_done,
                    self._medias_total,
                    100 * self._medias_done // self._medias_total,
                )
            )

        if self._task_percent is not None:
            msg.append(_("current task %d%%") % self._task_percent)

        return _("Progress: %s") % ", ".join(msg)

    def updated(self):
        pass


class Album(object):

    def __init__(self, source_dir, config=None):
        self.source_dir = os.path.abspath(source_dir)

        self.config = LazygalConfig(load_defaults=True)

        logging.debug(_("Trying loading user config %s"), USER_CONFIG_PATH)
        self.config.load_any(USER_CONFIG_PATH)

        sourcedir_configfile = os.path.join(source_dir, SOURCEDIR_CONFIGFILE)
        if os.path.isfile(sourcedir_configfile):
            logging.debug(_("Loading root config %s"), sourcedir_configfile)
            try:
                self.config.load_any(sourcedir_configfile)
            except LazygalConfigDeprecated:
                logging.error(
                    _(
                        "'%s' uses a deprecated syntax: please refer to lazygal.conf(5) manual page."
                    ),
                    sourcedir_configfile,
                )
                sys.exit(1)
        if config is not None:  # Supplied config
            self.config.load(config)

        if self.config.get("runtime", "quiet"):
            logging.getLogger().setLevel(logging.ERROR)
        if self.config.get("runtime", "debug"):
            logging.getLogger().setLevel(logging.DEBUG)
            GExiv2.log_set_level(GExiv2.LogLevel.INFO)

        self.clean_dest = self.config.get("global", "clean-destination")
        self.preserves = self.config.get("global", "preserve") + self.config.get(
            "global", "preserve_args"
        )
        self.force_gen_pages = self.config.get("global", "force-gen-pages")

        self.set_theme(self.config.get("global", "theme"))
        self.excludes = self.config.get("global", "exclude") + self.config.get(
            "global", "exclude_args"
        )

        self.dir_flattening_depth = self.config.get("global", "dir-flattening-depth")

        self.__statistics = None

    def set_theme(self, theme_name=theme.DEFAULT_THEME):
        self.theme = theme.Theme(os.path.join(DATAPATH, "themes"), theme_name)
        self.theme.prepare_tpl_loader(tpl.TplFactory)
        self.theme.check_shared_files()

    def _str_humanize(self, text):
        dash_replaced = text.replace("_", " ")
        return dash_replaced

    def is_in_sourcetree(self, path):
        return pathutils.is_subdir_of(self.source_dir, path)

    def cleanup(self, file_path, context_path):
        if self.clean_dest:
            # Do not delete something out of dest-dir.
            assert pathutils.is_subdir_of(context_path, file_path)
            tail = os.path.basename(file_path)
            for pattern in self.preserves:
                # Do not delete something the user wants to keep.
                if fnmatch.fnmatch(tail, pattern):
                    logging.info("  PRESERVE %s", tail)
                    return
            if os.path.isdir(file_path):
                shutil.rmtree(file_path)
            else:
                os.unlink(file_path)
            logging.info("  RM %s", file_path)

    def generate_default_metadata(self):
        """
        Generate default metadata files if no exists.
        """
        logging.debug(_("Generating metadata in %s"), self.source_dir)

        for root, dirnames, filenames in pathutils.walk(self.source_dir):
            filenames.sort()  # This is required for the ignored files
            # checks to be reliable.
            source_dir = sourcetree.Directory(root, [], filenames, self)
            logging.info(_("[Entering %%ALBUMROOT%%/%s]"), source_dir.strip_root())
            logging.debug("(%s)", source_dir.path)

            metadata.DefaultMetadata(source_dir, self).make()

    def stats(self):
        if self.__statistics is None:
            self.__statistics = {"total": 0, "bydir": {}}
            for root, dirnames, filenames in pathutils.walk(self.source_dir):
                dir_medias = len(
                    [
                        f
                        for f in filenames
                        if sourcetree.MediaHandler.is_known_media(f, self)
                    ]
                )
                self.__statistics["total"] = self.__statistics["total"] + dir_medias
                self.__statistics["bydir"][root] = dir_medias
        return self.__statistics

    def generate(self, dest_dir=None, progress=None):
        if dest_dir is None:
            dest_dir = self.config.get("global", "output-directory")
        sane_dest_dir = os.path.abspath(os.path.expanduser(dest_dir))

        if not progress:
            progress = DummyProgress()

        pub_url = self.config.get("global", "puburl")
        check_all_dirs = self.config.get("runtime", "check-all-dirs")

        if self.is_in_sourcetree(sane_dest_dir):
            raise ValueError(
                _("Fatal error, web gallery directory is within source tree.")
            )

        logging.debug(_("Generating to %s"), sane_dest_dir)

        if pub_url:
            feed = genpage.WebalbumFeed(self, sane_dest_dir, pub_url)
        else:
            feed = None

        dir_heap = {}
        for root, dirnames, filenames in pathutils.walk(self.source_dir):

            if root in dir_heap:
                subdirs, subgals = dir_heap[root]
                del dir_heap[root]  # No need to keep it there
            else:
                subdirs = []
                subgals = []

            checked_dir = sourcetree.File(root, self)

            if checked_dir.should_be_skipped():
                logging.debug(_("(%s) has been skipped"), checked_dir.path)
                continue
            if checked_dir.path == os.path.join(
                sane_dest_dir, DEST_SHARED_DIRECTORY_NAME
            ):
                logging.error(
                    _(
                        "(%s) has been skipped because its name collides with the shared material directory name"
                    ),
                    checked_dir.path,
                )
                continue

            logging.info(_("[Entering %%ALBUMROOT%%/%s]"), checked_dir.strip_root())
            logging.debug("(%s)", checked_dir.path)

            source_dir = sourcetree.Directory(root, subdirs, filenames, self)

            destgal = WebalbumDir(source_dir, subgals, self, sane_dest_dir, progress)

            if source_dir.is_album_root():
                # Use root config tpl vars for shared files
                tpl_vars = destgal.tpl_vars

            if not source_dir.is_album_root():
                container_dirname = os.path.dirname(root)
                if container_dirname not in dir_heap:
                    dir_heap[container_dirname] = ([], [])
                container_subdirs, container_subgals = dir_heap[container_dirname]
                container_subdirs.append(source_dir)
                container_subgals.append(destgal)

            if feed and source_dir.is_album_root():
                feed.set_title(source_dir.human_name)
                md = destgal.source_dir.metadata.get()
                if "album_description" in md.keys():
                    feed.set_description(md["album_description"])
                destgal.register_output(feed.path)

            if feed:
                feed.push_dir(destgal)
                destgal.register_feed(feed)

            if check_all_dirs or destgal.needs_build_quick():
                destgal.make()
            else:
                progress.media_done(self.stats()["bydir"][destgal.source_dir.path])
                logging.info(
                    _(
                        "  SKIPPED because of mtime, touch source or use --check-all-dirs to override."
                    )
                )

            # Force some memory cleanups, this is usefull for big albums.
            del destgal
            gc.collect()

            progress.dir_done()

            logging.info(_("[Leaving  %%ALBUMROOT%%/%s]"), source_dir.strip_root())

        if feed:
            feed.make()

        # Force to check for unexpected files
        SharedFiles(self, sane_dest_dir, tpl_vars).make(True)


# vim: ts=4 sw=4 expandtab



# Deejayd, a media player daemon
# Copyright (C) 2013 Mickael Royer <mickael.royer@gmail.com>
#                    Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import sys
import posixpath
import logging
import urllib.parse as urlparse


def is_root_posix(path):
    return path == "/"


def is_root_win32(path):
    return path[1:] == ":\\"  # strip drive letter in comparison


if sys.platform == "win32":
    is_root = is_root_win32
else:
    is_root = is_root_posix


def is_subdir_of(dir_path, path):
    """
    Returns whether path is a subdirectory of dir_path.
    """
    dir_path = os.path.abspath(dir_path)
    path = os.path.abspath(path)
    test_path = path
    while test_path != dir_path and not is_root(test_path):
        test_path, tail = os.path.split(test_path)
        if test_path == "":
            raise RuntimeError("subdir test failure: please report a bug")

    if test_path == dir_path:
        return True
    else:
        return False


def url_path(physical_path, input_pathmodule=os.path):
    """
    Convert a physical path to a path suitable for use in a URL link,
    i.e. using forward slashes. This can only be used for relative paths
    because while converting, the root (either '/' or 'C:\\') is irrelevant.
    """
    if input_pathmodule == posixpath:
        return physical_path

    head = physical_path
    path_list = []
    while head != "" and not is_root_posix(head) and not is_root_win32(head):
        head, tail = input_pathmodule.split(head)
        path_list.append(tail)

    if path_list == []:
        return ""

    path_list.reverse()
    return posixpath.join(*path_list)


def url_quote(url, anchor=""):
    tokens = []
    if url.startswith("http://") or url.startswith("https://"):
        parsed = urlparse.urlparse(url)
        if parsed.scheme:
            tokens.append(parsed.scheme)
            tokens.append("://")
            tokens.append(parsed.netloc)
        toquote = parsed.path
    else:
        toquote = url

    tokens.append(urlparse.quote(toquote.encode(sys.getfilesystemencoding()), "/~"))

    if anchor:
        tokens.append("#")
        tokens.append(urlparse.quote(anchor.encode(sys.getfilesystemencoding())))

    return "".join(tokens)


def walk(top, walked=None, topdown=False):
    """
    This is a wrapper around os.walk() from the standard library:
    - following symbolic links on directories
    - whith barriers in place against walking twice the same directory,
      which may happen when two directory trees have symbolic links to
      each other's contents.
    """
    if walked is None:
        walked = []

    for root, dirs, files in os.walk(top, topdown=topdown):
        walked.append(os.path.realpath(root))

        # Follow symlinks if they have not been walked yet
        for d in dirs:
            d_path = os.path.join(root, d)
            if os.path.islink(d_path):
                if os.path.realpath(d_path) not in walked:
                    for x in walk(d_path, walked):
                        yield x
                else:
                    logging.error(
                        "Not following symlink '%s' because directory has already been processed.",
                        d_path,
                    )

        yield root, dirs, files


def walk_and_do(top=None, walked=None, dcb=None, fcb=None, topdown=False):
    """
    This walk calls dcb on each found directory and fcb on each found
    file with the following arguments :
        - path
    """
    for root, dirs, files in walk(top, walked, topdown=topdown):
        if dcb is not None:
            dcb(root, dirs, files)
        if fcb is not None:
            for f in files:
                fcb(os.path.join(root, f))


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import logging
import configparser
import collections
import functools
import json
import copy


USER_CONFIG_PATH = os.path.expanduser("~/.lazygal/config")
DEFAULT_CONFIG_PATH = os.path.join(os.path.dirname(__file__), "defaults.json")


class BetterConfigParser(configparser.RawConfigParser):

    def getint(self, section, option):
        try:
            if not self.getboolean(section, option):
                return False
            else:
                raise ValueError
        except (ValueError, AttributeError):
            return configparser.RawConfigParser.getint(self, section, option)

    def getstr(self, section, option):
        try:
            if not self.getboolean(section, option):
                return False
            else:
                raise ValueError
        except (ValueError, AttributeError):
            return configparser.RawConfigParser.get(self, section, option)

    def getlist(self, section, option):
        str_vlist = self.get(section, option)

        if str_vlist == "":
            return []

        if type(str_vlist) is not list:
            str_vlist = str_vlist.split(",")

        # handle the case several vals were given at once (separated by comas)
        vlist = []
        for v in str_vlist:
            multiple_v = v.split(",")
            for mv in multiple_v:
                vlist.append(mv)

        return vlist

    def load(self, other_config, init=False, sections=None):
        """
        Take another configuration object and overload values in this config
        object.
        """
        all_sections = False
        if sections is None:
            sections = other_config.sections()
            all_sections = True

        for section in sections:
            if all_sections or other_config.has_section(section):
                if not self.has_section(section):
                    if not init:
                        self.new_section_cb(section)
                    self.add_section(section)
                for option in other_config.options(section):
                    if not init and not self.has_option(section, option):
                        self.new_option_cb(section, option)
                    self.set(section, option, other_config.get(section, option))

    def new_section_cb(self, section):
        pass

    def new_option_cb(self, section, option):
        pass


class LazygalConfigDeprecated(BaseException):
    pass


class NoSectionError(BaseException):
    pass


class NoOptionError(BaseException):
    pass


class LazygalIniConfig(BetterConfigParser):

    def check_deprecation(self, config=None):
        if config is None:
            config = self

        if config.has_section("lazygal"):
            raise LazygalConfigDeprecated("'lazygal' section is deprecated")

    def read(self, filenames):
        conf = BetterConfigParser()
        conf.read(filenames)
        self.load(conf)
        self.check_deprecation()

    def load(self, other_config, init=False, sections=None):
        self.check_deprecation(other_config)
        BetterConfigParser.load(self, other_config, init, sections)

    def new_section_cb(self, section):
        if section != "template-vars":
            logging.warning(_("  Ignoring unknown section '%s'."), section)

    def new_option_cb(self, section, option):
        if section != "template-vars":
            logging.warning(
                _("  Ignoring unknown option '%s' in section '%s'."), option, section
            )


def false_or(s, f=None):
    if s.lower() in ("no", "false"):
        return False
    elif f:
        return f(s)
    else:
        return s


def get_bool(s):
    if s.lower() in ("true", "yes", "on", "1") or s == 1:
        return True
    if s.lower() in ("false", "no", "off", "0") or s == 0:
        return False
    raise ValueError(_("Unknown boolean value '%s'") % s)


def get_list(s):
    if s:
        return [i.strip() for i in s.split(",")]
    return []


def get_dict(s):
    d = {}
    for i in s.split(","):
        key, value = i.split("=")
        d[key.strip()] = value.strip()
    return d


def get_int(s):
    return int(s)


def get_order(s):
    try:
        order, reverse = s.split(":")
    except ValueError:
        order, reverse = s, False
    return {"order": order, "reverse": reverse == "reverse"}


def get_image_size(s):
    sizes_defs_str = get_list(s)
    sizes_defs = []
    first = True
    for size_defs in sizes_defs_str:
        try:
            name, defs = size_defs.split("=")
        except ValueError:
            raise ValueError(
                _(
                    'Sizes is a comma-separated list of size names and specs:\n\t e.g. "small=640x480,medium=1024x768".'
                )
            )

        sizes_defs.append({"name": name, "defs": defs, "default": first})
        first = False
    return sizes_defs


STRING_TO_JSON = collections.OrderedDict(
    {
        "runtime": {
            "quiet": get_bool,
            "debug": get_bool,
            "check-all-dirs": get_bool,
        },
        "global": {
            "force-gen-pages": get_bool,
            "clean-destination": get_bool,
            "preserve": get_list,
            "dir-flattening-depth": functools.partial(false_or, f=get_int),
            "puburl": false_or,
            "exclude": get_list,
            "preserve_args": get_list,
            "exclude_args": get_list,
        },
        "webgal": {
            "image-size": get_image_size,
            "thumbs-per-page": get_int,
            "filter-by-tag": get_list,
            "sort-medias": get_order,
            "sort-subgals": get_order,
            "original": get_bool,
            "original-baseurl": false_or,
            "original-symlink": get_bool,
            "dirzip": get_bool,
            "jpeg-quality": get_int,
            "jpeg-optimize": get_bool,
            "jpeg-progressive": get_bool,
            "publish-metadata": get_bool,
            "keep-gps": get_bool,
        },
    }
)


class LazygalConfig(object):
    valid_sections = (
        "runtime",
        "global",
        "webgal",
        "template-vars",
    )

    def __init__(self, load_defaults=False):
        self.c = collections.OrderedDict()
        self.valid_options = {s: [] for s in self.valid_sections}

        self.load(
            self.load_file(DEFAULT_CONFIG_PATH), defaults=True, setvalue=load_defaults
        )
        self.files = [DEFAULT_CONFIG_PATH]

    def has_section(self, section):
        return section in self.c

    def add_section(self, section):
        if not self.has_section(section):
            if section in self.valid_sections:
                self.c[section] = collections.OrderedDict()
            else:
                raise ValueError("section '%s' is not a valid section name" % section)

    def options(self, section):
        return self.c[section].keys()

    def get(self, section, option):
        if not self.has_section(section):
            raise NoSectionError(section)
        if option in self.c[section]:
            return self.c[section][option]
        else:
            raise NoOptionError(_("%s in section %s") % (option, section))

    def set(self, section, option, value):
        self.add_section(section)

        if option in self.valid_options[section] or section == "template-vars":
            if (
                type(value) is str
                and section in STRING_TO_JSON
                and option in STRING_TO_JSON[section]
            ):
                value = STRING_TO_JSON[section][option](value)
            self.c[section][option] = copy.deepcopy(value)
        else:
            raise ValueError(
                _("option '%s' is not valid in section '%s'") % (option, section)
            )

    def load(self, newconf, setvalue=True, defaults=False):
        for section, s in newconf.items():
            if defaults and section not in self.valid_sections:
                continue  # ignore default section loading

            for k, v in s.items():
                if defaults:
                    self.valid_options[section].append(k)
                if not defaults or (defaults and setvalue):
                    try:
                        self.set(section, k, v)
                    except ValueError as e:
                        logging.warning(_("Ignoring option: %s") % e.args[0])

    def load_file(self, path):
        newconf = None
        with open(path, "r") as json_fp:
            newconf = json.load(json_fp)
        return newconf

    def load_inifile(self, path):
        iniconf = LazygalIniConfig(defaults=False)
        with open(path) as fp:
            iniconf.read_file(fp)

        for s in iniconf.sections():
            for key, value in iniconf.items(s):
                try:
                    self.set(s, key, value)
                except ValueError as e:
                    logging.warning(_("Ignoring option: %s") % e.args[0])

    def load_any(self, path):
        if not os.path.isfile(path):
            logging.debug(_("Cannot load non-existent config %s.") % path)
            return

        try:
            self.load(self.load_file(path))
        except ValueError:
            try:
                self.load_inifile(path)
            except Exception:
                raise
            else:
                logging.warning(_("INI-style config file format is deprecated."))
        else:
            self.files.append(path)

    def __str__(self):
        return json.dumps(self.c)

    def items(self):
        return self.c.items()

    def __getitem__(self, key):
        return self.c[key]


class LazygalWebgalConfig(LazygalConfig):
    valid_sections = (
        "webgal",
        "template-vars",
    )


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import email.utils
import os
import sys
from xml.etree import ElementTree as ET


from . import pathutils


class RSS20(object):

    def __init__(self, link, maxitems=10):
        self.items = []

        self.title = None
        self.description = None
        self.link = link
        self.__maxitems = maxitems

    def __get_root_and_channel(self, feed_filename):
        root = ET.Element(
            "rss", {"version": "2.0", "xmlns:atom": "http://www.w3.org/2005/Atom"}
        )

        channel = ET.SubElement(root, "channel")
        ET.SubElement(channel, "title").text = self.title
        ET.SubElement(channel, "link").text = pathutils.url_quote(self.link)
        ET.SubElement(channel, "description").text = self.description
        ET.SubElement(
            channel,
            "atom:link",
            {
                "href": self.link + feed_filename,
                "rel": "self",
                "type": "application/rss+xml",
            },
        )

        return root, channel

    def __item_older(self, x, y):
        return int(y["timestamp"] - x["timestamp"])

    def push_item(self, title, link, contents, timestamp):
        item = {}
        item["title"] = title
        item["link"] = link
        item["contents"] = contents
        item["timestamp"] = timestamp

        i = 0
        while i < len(self.items) and self.__item_older(item, self.items[i]) > 0:
            i = i + 1

        if i < self.__maxitems:
            # We have a candidate
            self.items.insert(i, item)
            if len(self.items) > self.__maxitems:
                # We have one too much, so get rid of it
                self.items.pop()

    def dump(self, path):
        (root, channel) = self.__get_root_and_channel(os.path.basename(path))

        pubdate = ET.SubElement(channel, "pubDate")

        self.items.sort(key=lambda x: int(x["timestamp"]))
        for item in self.items:
            rssitem = ET.SubElement(channel, "item")
            ET.SubElement(rssitem, "title").text = item["title"]
            ET.SubElement(rssitem, "link").text = pathutils.url_quote(item["link"])
            ET.SubElement(rssitem, "guid").text = pathutils.url_quote(item["link"])
            date = email.utils.formatdate(item["timestamp"], localtime=True)
            ET.SubElement(rssitem, "pubDate").text = date
            ET.SubElement(rssitem, "description").text = item["contents"]

        pubdate.text = email.utils.formatdate(localtime=True)

        feedtree = ET.ElementTree(root)
        feedtree.write(path, "utf-8")


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
import datetime
import shutil
import json

from PIL import Image

from . import LazygalTestGen, has_symlinks
import lazygal.config
from lazygal.generators import WebalbumDir
from lazygal.sourcetree import Directory
from lazygal.metadata import GEXIV2_DATE_FORMAT, GExiv2
from lazygal.mediautils import VideoProcessor, HAVE_VIDEO


class TestGenerators(LazygalTestGen):

    def test_albumstats(self):
        pics = ["img%d.jpg" % i for i in range(0, 8)]
        self.setup_subgal("subgal", pics)
        self.setup_subgal("vidgal", ["vid.webm"])

        expected_stats = {
            self.source_dir: 0,
            os.path.join(self.source_dir, "subgal"): 8,
            os.path.join(self.source_dir, "vidgal"): 1,
        }

        self.assertEqual(self.album.stats()["total"], 9)
        self.assertEqual(self.album.stats()["bydir"], expected_stats)

    def test_genfile_filelayout(self):
        source_subgal = self.setup_subgal("subgal", ["subgal_img.jpg"])

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        error = "%s has not been generate though it should have"

        # Check root dir contents
        self.assertTrue(os.path.isdir(dest_path))
        for fn in ("index.json", "index.html", "index_medium.html"):
            self.assertTrue(os.path.isfile(os.path.join(dest_path, fn)), error % fn)

        # Check subgal dir contents
        dest_subgal_path = os.path.join(dest_path, "subgal")
        self.assertTrue(os.path.isdir(dest_subgal_path))
        for fn in (
            "index.json",
            "index.html",
            "index_medium.html",
            "subgal_img.html",
            "subgal_img_medium.html",
            "subgal_img_thumb.jpg",
            "subgal_img_small.jpg",
            "subgal_img_medium.jpg",
        ):
            self.assertTrue(
                os.path.isfile(os.path.join(dest_subgal_path, fn)), error % fn
            )

        # Check JSON root index
        with open(os.path.join(dest_path, "index.json")) as json_fp:
            pindex = json.load(json_fp)
            self.assertEqual(pindex["medias"], {})
            self.assertEqual(
                pindex["count"], {"media": 0, "video": 0, "image": 0, "subgal": 1}
            )
            self.assertEqual(pindex["all_count"], {"media": 1, "video": 0, "image": 1})
            self.assertEqual(pindex["subgals"], ["subgal"])

        # Check JSON subgal index
        with open(os.path.join(dest_path, "subgal", "index.json")) as json_fp:
            pindex = json.load(json_fp)
            self.assertEqual(
                pindex["medias"],
                {
                    "subgal_img.jpg": {
                        "date": "2010-02-05T23:56:24",
                        "width": 640,
                        "height": 427,
                        "alphachannel": False,
                        "metadata": {
                            "date": "2010-02-05T23:56:24",
                            "comment": None,
                            "rotation": 0,
                            "camera_name": "HTC Dream",
                            "lens_name": "",
                            "flash": "",
                            "exposure": "None",
                            "iso": "",
                            "fnumber": "",
                            "focal_length": "",
                            "authorship": "",
                            "keywords": [],
                            "location": None,
                        },
                        "type": "image",
                    }
                },
            )
            self.assertEqual(
                pindex["count"], {"media": 1, "video": 0, "image": 1, "subgal": 0}
            )
            self.assertEqual(pindex["all_count"], {"media": 1, "video": 0, "image": 1})
            self.assertEqual(pindex["subgals"], [])

    def test_genfile_umask(self):
        """
        The software should honor the umask setting.
        """
        oldmask = os.umask(0o027)

        source_subgal = self.setup_subgal("subgal", ["subgal_img.jpg"])

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        target_dperms = oct(0o040750)
        target_fperms = oct(0o0100640)

        error = "wrong perms %s instead of %s for %s"

        for root, dir, files in os.walk(dest_path):
            for f in files:
                fpath = os.path.join(root, f)
                fperms = oct(os.stat(fpath).st_mode)
                self.assertEqual(
                    fperms, target_fperms, error % (fperms, target_fperms, fpath)
                )

        for d in ("subgal", "shared"):
            dpath = os.path.join(dest_path, d)
            dperms = oct(os.stat(dpath).st_mode)
            self.assertEqual(
                dperms, target_dperms, error % (dperms, target_dperms, dpath)
            )

        os.umask(oldmask)

    def test_spot_foreign_files(self):
        """
        Files that are not part of what was generated or updated shall be
        spotted.
        """
        pics = ["img%d.jpg" % i for i in range(0, 8)]
        source_subgal = self.setup_subgal("subgal", pics)

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        # add a thumbs that should not be there
        self.add_img(dest_path, "extra_thumb.jpg")
        self.add_img(os.path.join(dest_path, "subgal"), "extra_thumb2.jpg")
        os.mkdir(os.path.join(dest_path, "extra_dir"))

        # remove a pic in source_dir
        os.unlink(os.path.join(self.source_dir, "subgal", "img6.jpg"))

        # new objects to probe filesystem
        pics.remove("img6.jpg")
        source_subgal = Directory(
            os.path.join(self.source_dir, "subgal"), [], pics, self.album
        )
        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)
        expected = map(
            lambda fn: os.path.join(dest_path, "subgal", fn),
            [
                "extra_thumb2.jpg",
                "img6_thumb.jpg",
                "img6_small.jpg",
                "img6_medium.jpg",
                "img6.html",
                "img6_medium.html",
            ],
        )
        self.assertEqual(sorted(dest_subgal.list_foreign_files()), sorted(expected))
        source_gal = Directory(self.source_dir, [source_subgal], [], self.album)
        dest_gal = WebalbumDir(source_gal, [dest_subgal], self.album, dest_path)
        expected = map(
            lambda fn: os.path.join(dest_path, fn), ["extra_thumb.jpg", "extra_dir"]
        )
        self.assertEqual(sorted(dest_gal.list_foreign_files()), sorted(expected))

    def test_cleanup(self):
        """
        Check that the foreign files are deleted.
        """
        config = lazygal.config.LazygalConfig()
        config.set("global", "clean-destination", "true")
        self.setup_album(config)

        pics = ["img%02d.jpg" % i for i in range(4, 8)]
        source_subgal = self.setup_subgal("subgal", pics)

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        # add thumbs and directories that should not be there
        self.add_img(dest_path, "extra_thumb.jpg")
        self.add_img(os.path.join(dest_path, "subgal"), "extra_thumb2.jpg")
        os.mkdir(os.path.join(dest_path, "extra_dir"))
        self.add_img(os.path.join(dest_path, "extra_dir"), "extra_thumb3.jpg")

        # remove a pic in source_dir
        os.unlink(os.path.join(self.source_dir, "subgal", "img06.jpg"))

        self.album.generate(dest_path)

        try:
            for f in [
                "extra_thumb2.jpg",
                "img06_thumb.jpg",
                "img06_small.jpg",
                "img06_medium.jpg",
                "img06.html",
                "img06_medium.html",
            ]:
                self.assertFalse(os.path.isfile(os.path.join(dest_path, "subgal", f)))

            for f in ["extra_thumb.jpg"]:
                self.assertFalse(os.path.isfile(os.path.join(dest_path, f)))
            for f in ["extra_dir"]:
                self.assertFalse(os.path.isdir(os.path.join(dest_path, f)))
        except AssertionError:
            print("\n contents of dest_path : ")
            print(sorted(os.listdir(dest_path)))
            print(sorted(os.listdir(os.path.join(dest_path, "subgal"))))
            raise

    def test_clean_empty_dirs(self):
        """
        Check that empty dirs at destination are removed
        .
        This case might happen when (1) the destination dir is created according
        to the contents of source_dir.
        (2) Then source_dir becomes empty, either because all the images have
        been removed, or because tag filtering is applied with different
        settings. The next lazygal generation should remove the destination dir.
        """
        config = lazygal.config.LazygalConfig()
        config.set("global", "clean-destination", "true")
        self.setup_album(config)

        pics = ["img.jpg"]
        source_subgal = self.setup_subgal("subgal", pics)

        dest_path = self.get_working_path()
        self.album.generate(dest_path)

        # all the files in subgal should be here after the first generation
        try:
            for f in [
                "img_thumb.jpg",
                "img_small.jpg",
                "img_medium.jpg",
                "img.html",
                "img_medium.html",
            ]:
                self.assertTrue(os.path.isfile(os.path.join(dest_path, "subgal", f)))

            for f in ["subgal"]:
                self.assertTrue(os.path.isdir(os.path.join(dest_path, f)))
        except AssertionError:
            print("\n contents of dest_path after first generation: ")
            print(sorted(os.listdir(dest_path)))
            print(sorted(os.listdir(os.path.join(dest_path, "subgal"))))
            raise

        # remove the pic in subgal, and force a new generation
        os.unlink(os.path.join(self.source_dir, "subgal", "img.jpg"))
        self.album.generate(dest_path)

        # now the subdirectory subgal and its contents should no longer be there
        try:
            for f in [
                "img.jpg",
                "img_thumb.jpg",
                "img_small.jpg",
                "img_medium.jpg",
                "img.html",
                "img_medium.html",
            ]:
                self.assertFalse(os.path.isfile(os.path.join(dest_path, "subgal", f)))

            for f in ["subgal"]:
                self.assertFalse(os.path.isdir(os.path.join(dest_path, f)))
        except AssertionError:
            print("\n contents of dest_path after the second generation: ")
            print(sorted(os.listdir(dest_path)))
            print(sorted(os.listdir(os.path.join(dest_path, "subgal"))))
            raise

    @unittest.skipIf(not has_symlinks(), "symlinks not supported on platform")
    def test_clean_dirsymlinks(self):
        """
        Check that symlinks in directory source, that should make albums
        in dest, are not cleaned up in dest.
        """
        config = lazygal.config.LazygalConfig()
        config.set("global", "clean-destination", "true")
        self.setup_album(config)

        pics = ["img.jpg"]
        source_subgal = self.setup_subgal("subgal", pics)

        # add a symlink on source that should be duplicated in dest
        out_of_tree_subgal = os.path.join(self.get_working_path(), "oot_subgal")
        os.mkdir(out_of_tree_subgal)
        self.add_img(out_of_tree_subgal, "oot_img.jpg")
        os.symlink(
            out_of_tree_subgal, os.path.join(self.source_dir, "oot_subgal_symlink")
        )

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        # all the files in subgal should be here after the first generation
        try:
            self.assertTrue(os.path.isdir(os.path.join(dest_path, "subgal")))
            for f in [
                "img_thumb.jpg",
                "img_small.jpg",
                "img_medium.jpg",
                "img.html",
                "img_medium.html",
            ]:
                self.assertTrue(os.path.isfile(os.path.join(dest_path, "subgal", f)))
            self.assertTrue(
                os.path.isdir(os.path.join(dest_path, "oot_subgal_symlink"))
            )
            for f in [
                "oot_img_thumb.jpg",
                "oot_img_small.jpg",
                "oot_img_medium.jpg",
                "oot_img.html",
                "oot_img_medium.html",
            ]:
                self.assertTrue(
                    os.path.isfile(os.path.join(dest_path, "oot_subgal_symlink", f))
                )
        except AssertionError:
            print("\n contents of dest_path after first generation: ")
            print(sorted(os.listdir(dest_path)))
            print(sorted(os.listdir(os.path.join(dest_path, "subgal"))))
            raise

        # remove the pic in subgal, and force a new generation
        os.unlink(os.path.join(self.source_dir, "subgal", "img.jpg"))
        os.rmdir(os.path.join(self.source_dir, "subgal"))
        self.album.generate(dest_path)

        # only file in symlinkd gal should be there
        try:
            self.assertFalse(os.path.exists(os.path.join(dest_path, "subgal")))
            for f in [
                "img_thumb.jpg",
                "img_small.jpg",
                "img_medium.jpg",
                "img.html",
                "img_medium.html",
            ]:
                self.assertFalse(os.path.exists(os.path.join(dest_path, "subgal", f)))
            self.assertTrue(
                os.path.isdir(os.path.join(dest_path, "oot_subgal_symlink"))
            )
            for f in [
                "oot_img_thumb.jpg",
                "oot_img_small.jpg",
                "oot_img_medium.jpg",
                "oot_img.html",
                "oot_img_medium.html",
            ]:
                self.assertTrue(
                    os.path.isfile(os.path.join(dest_path, "oot_subgal_symlink", f))
                )
        except AssertionError:
            print("\n contents of dest_path after first generation: ")
            print(sorted(os.listdir(dest_path)))
            print(sorted(os.listdir(os.path.join(dest_path, "subgal"))))
            raise

    @unittest.skipIf(not has_symlinks(), "symlinks not supported on platform")
    def test_originals_symlinks(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "original", "Yes")
        config.set("webgal", "original-symlink", "Yes")
        self.setup_album(config)

        img_path = self.add_img(self.source_dir, "symlink_target.jpg")

        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        symlink = os.path.join(dest_dir, os.path.basename(img_path))
        # Test if the original in the webgal is a symlink
        self.assertTrue(os.path.islink(symlink))
        # Test if that symlink point to the image in the source_dir
        self.assertEqual(os.path.realpath(symlink), img_path)

    def test_metadata_osize_copy(self):
        img_path = self.add_img(self.source_dir, "md_filled.jpg")

        # Add some metadata
        gps_data = GExiv2.Metadata(self.get_sample_path("sample-with-gps.jpg"))
        source_image = GExiv2.Metadata(img_path)
        for tag in gps_data.get_exif_tags():
            source_image[tag] = gps_data[tag]
        dummy_comment = "nice photo"
        source_image["Exif.Photo.UserComment"] = dummy_comment
        dummy_date = datetime.datetime(2011, 2, 3, 12, 51, 43)
        source_image["Exif.Photo.DateTimeDigitized"] = dummy_date.strftime(
            GEXIV2_DATE_FORMAT
        )
        assert "Exif.GPSInfo.GPSLongitude" in source_image
        assert "Exif.GPSInfo.GPSLatitude" in source_image
        source_image.save_file()

        # Generate album
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        dest_img_path = os.path.join(dest_dir, "md_filled_small.jpg")
        dest_image = GExiv2.Metadata(dest_img_path)

        # Check that metadata is still here for reduced pictures.
        self.assertEqual(dest_image["Exif.Photo.UserComment"], dummy_comment)
        self.assertEqual(
            dest_image["Exif.Photo.DateTimeDigitized"],
            dummy_date.strftime("%Y:%m:%d %H:%M:%S"),
        )

        # Check that blacklised tags are not present anymore in the reduced
        # picture.
        def lat():
            return dest_image["Exif.GPSInfo.GPSLongitude"]

        self.assertRaises(KeyError, lat)

        def long():
            return dest_image["Exif.GPSInfo.GPSLatitude"]

        self.assertRaises(KeyError, long)

        # Check that metadata is not in the JSON index
        with open(os.path.join(dest_dir, "index.json")) as json_fp:
            pindex = json.load(json_fp)
            self.assertEqual(
                pindex["medias"]["md_filled.jpg"]["metadata"]["location"], None
            )

    def test_metadata_osize_nopublish(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "publish-metadata", "No")
        self.setup_album(config)

        img_path = self.add_img(self.source_dir, "md_filled.jpg")

        # Add some metadata
        source_image = GExiv2.Metadata(img_path)
        dummy_comment = "nice photo"
        source_image["Exif.Photo.UserComment"] = dummy_comment
        source_image.save_file()

        # Generate album
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        dest_img_path = os.path.join(dest_dir, "md_filled_small.jpg")
        dest_image = GExiv2.Metadata(dest_img_path)

        # Check that metadata is not here for reduced pictures.
        def com():
            return dest_image["Exif.Photo.UserComment"]

        self.assertRaises(KeyError, com)

        # Check that metadata is not in the JSON index
        with open(os.path.join(dest_dir, "index.json")) as json_fp:
            pindex = json.load(json_fp)
            self.assertFalse(pindex["medias"]["md_filled.jpg"]["metadata"])

    def test_resize_rotate_size(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "image-size", "std=800x600")
        self.setup_album(config)

        norotate_path = self.add_img(self.source_dir, "norotate.jpg")
        torotate_path = self.add_img(self.source_dir, "torotate.jpg")
        torotate = GExiv2.Metadata(torotate_path)
        torotate["Exif.Image.Orientation"] = "8"
        torotate.save_file()

        # Generate album
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        dest_norotate_path = os.path.join(dest_dir, "norotate_std.jpg")
        im = Image.open(dest_norotate_path)
        self.assertEqual(
            im.size,
            (
                800,
                533,
            ),
        )
        im.close()

        dest_torotate_path = os.path.join(dest_dir, "torotate_std.jpg")
        im = Image.open(dest_torotate_path)
        self.assertEqual(
            im.size,
            (
                400,
                600,
            ),
        )
        im.close()

    def test_sizeasoriginal(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "original", True)
        config.set("webgal", "image-size", "std=0x0")
        self.setup_album(config)

        self.add_img(self.source_dir, "img.jpg")

        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        shouldbeoriginal = os.path.join(dest_dir, "img.jpg")
        im = Image.open(shouldbeoriginal)
        self.assertEqual(
            im.size,
            (
                640,
                427,
            ),
        )
        im.close()

    def test_image_with_alpha(self):
        im = Image.new("RGBA", (100, 100))
        im.save(os.path.join(self.source_dir, "with-alpha.png"))

        dest_path = self.get_working_path()
        self.album.generate(dest_path)

    def test_webalbumpic_bg(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "webalbumpic-bg", "black")
        self.setup_album(config)

        self.add_img(self.source_dir, "img.jpg")

        dest_path = self.get_working_path()
        self.album.generate(dest_path)

    def test_feed(self):
        config = lazygal.config.LazygalConfig()
        config.set("global", "puburl", "http://example.com/album/")
        self.setup_album(config)

        img_path = self.add_img(self.source_dir, "img01.jpg")
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        self.assertTrue(os.path.isfile(os.path.join(dest_dir, "index.xml")))

    def test_dirzip(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "dirzip", "Yes")
        self.setup_album(config)

        img_path = self.add_img(self.source_dir, "img01.jpg")
        img_path = self.add_img(self.source_dir, "img02.jpg")
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        self.assertTrue(os.path.isfile(os.path.join(dest_dir, "src.zip")))

    def test_filter_by_tag(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "filter-by-tag", "lazygal")
        self.setup_album(config)

        # tagfound pictures will be pushed on the destination, tagnotfound pictures
        # should be filtered out.
        # The sub-directory 'sdir_tagnotfound' should not be created on the destination
        # side, because it should be empty.
        tagfound_path = self.add_img(self.source_dir, "tagfound.jpg")
        tagfound2_path = self.add_img(self.source_dir, "tagfound2.jpg")
        tagnotfound_path = self.add_img(self.source_dir, "tagnotfound.jpg")
        untagged_path = self.add_img(self.source_dir, "untagged.jpg")
        sdir_tagfound = self.setup_subgal(
            "sdir_tagfound", ["sdir_tagfound.jpg", "sdir_tagnotfound.jpg"]
        )
        sdir_tagnotfound = self.setup_subgal(
            "sdir_tagnotfound", ["sdir_tagnotfound.jpg"]
        )
        sdir_untagged = self.setup_subgal("sdir_untagged", ["untagged.jpg"])
        tagfound_subdir_path = os.path.join(
            self.source_dir, sdir_tagfound.name, "sdir_tagfound.jpg"
        )
        tagnotfound_subdir_path = os.path.join(
            self.source_dir, sdir_tagnotfound.name, "sdir_tagnotfound.jpg"
        )
        tagfound = GExiv2.Metadata(tagfound_path)
        tagfound2 = GExiv2.Metadata(tagfound2_path)
        tagnotfound = GExiv2.Metadata(tagnotfound_path)
        tagfound_sd = GExiv2.Metadata(tagfound_subdir_path)
        tagnotfound_sd = GExiv2.Metadata(tagnotfound_subdir_path)
        tagfound["Iptc.Application2.Keywords"] = "lazygal"
        tagfound["Xmp.dc.subject"] = "lazygal2"
        tagfound.save_file()
        tagfound2["Iptc.Application2.Keywords"] = "lazygalagain"
        tagfound2["Xmp.dc.subject"] = "lazygal"
        tagfound2.save_file()
        tagnotfound["Iptc.Application2.Keywords"] = "another_tag"
        tagnotfound.save_file()
        tagfound_sd["Iptc.Application2.Keywords"] = "lazygal"
        tagfound_sd["Xmp.dc.subject"] = "lazygal2"
        tagfound_sd.save_file()
        tagnotfound_sd["Iptc.Application2.Keywords"] = "lazygal_lazygal"
        tagnotfound_sd["Xmp.dc.subject"] = "lazygal2"
        tagnotfound_sd.save_file()

        # generate album
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        try:
            self.assertTrue(os.path.isdir(os.path.join(dest_dir, "sdir_tagfound")))
            self.assertTrue(
                os.path.isfile(os.path.join(dest_dir, "tagfound_thumb.jpg"))
            )
            self.assertTrue(
                os.path.isfile(os.path.join(dest_dir, "tagfound2_thumb.jpg"))
            )
            self.assertFalse(
                os.path.isfile(os.path.join(dest_dir, "tagnotfound_thumb.jpg"))
            )
            self.assertTrue(
                os.path.isfile(
                    os.path.join(dest_dir, "sdir_tagfound", "sdir_tagfound_thumb.jpg")
                )
            )
            self.assertFalse(
                os.path.isfile(
                    os.path.join(
                        dest_dir, "sdir_tagnotfound", "sdir_tagnotfound_thumb.jpg"
                    )
                )
            )
            self.assertFalse(os.path.isdir(os.path.join(dest_dir, "sdir_tagnotfound")))
            self.assertFalse(
                os.path.isfile(os.path.join(dest_dir, "sdir_untagged", "untagged.jpg"))
            )
            self.assertFalse(os.path.isdir(os.path.join(dest_dir, "sdir_untagged")))
        except AssertionError:
            print("\n contents of dest_dir : ")
            print(os.listdir(dest_dir))
            raise

    def test_filter_and_dirzip(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "dirzip", "Yes")
        config.set("webgal", "filter-by-tag", "lazygal")
        self.setup_album(config)

        # We need at least two pictures to display, because lazygal generates a
        # zip archive only if there is more than one picture.
        tagfound_path = self.add_img(self.source_dir, "tagfound.jpg")
        tagfound_path2 = self.add_img(self.source_dir, "tagfound2.jpg")
        tagnotfound_path = self.add_img(self.source_dir, "tagnotfound.jpg")
        tagfound = GExiv2.Metadata(tagfound_path)
        tagfound2 = GExiv2.Metadata(tagfound_path2)
        tagnotfound = GExiv2.Metadata(tagnotfound_path)
        tagfound["Iptc.Application2.Keywords"] = "lazygal"
        tagfound["Xmp.dc.subject"] = "lazygal2"
        tagfound.save_file()
        tagfound2["Iptc.Application2.Keywords"] = "lazygalagain"
        tagfound2["Xmp.dc.subject"] = "lazygal"
        tagfound2.save_file()
        tagnotfound["Iptc.Application2.Keywords"] = "another_tag"
        tagnotfound.save_file()

        # generate album
        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)

        try:
            self.assertTrue(os.path.isfile(os.path.join(dest_dir, "src.zip")))
            self.assertTrue(
                os.path.isfile(os.path.join(dest_dir, "tagfound_thumb.jpg"))
            )
            self.assertTrue(
                os.path.isfile(os.path.join(dest_dir, "tagfound2_thumb.jpg"))
            )
            self.assertFalse(os.path.isfile(os.path.join(dest_dir, "false_thumb.jpg")))
        except AssertionError:
            print("\n contents of dest_dir : ")
            print(os.listdir(dest_dir))
            raise

    @unittest.skipIf(not HAVE_VIDEO, "video support not available")
    def test_withvideo(self):
        source_subgal = self.setup_subgal("subgal", [], ["vid.mov", "vid-silent.mov"])

        # silence a video to make sure it does not crash
        silencer = VideoProcessor(os.path.join(source_subgal.path, "vid.mov"))
        silencer.output_file_opts.extend(["-c", "copy", "-an"])
        # overwrite the existing file
        silencer.convert(os.path.join(source_subgal.path, "vid-silent.mov"))

        # create empty video file to ensure nothing crashes
        self.create_file(os.path.join(source_subgal.path, "vid-broken.mov"))

        dest_path = self.get_working_path()

        self.album.generate(dest_path)

        error = "%s has not been generated though it should have"

        # Check root dir contents
        self.assertTrue(os.path.isdir(dest_path))
        for fn in ("index.html", "index_medium.html"):
            self.assertTrue(os.path.isfile(os.path.join(dest_path, fn)), error % fn)

        # Check subgal dir contents
        dest_subgal_path = os.path.join(dest_path, "subgal")
        self.assertTrue(os.path.isdir(dest_subgal_path))
        for fn in (
            "index.html",
            "index_medium.html",
            "vid.html",
            "vid_medium.html",
            "vid_thumb.jpg",
            "vid_video.webm",
            "vid-silent.html",
            "vid-silent_medium.html",
            "vid-silent_thumb.jpg",
            "vid-silent_video.webm",
        ):
            self.assertTrue(
                os.path.isfile(os.path.join(dest_subgal_path, fn)), error % fn
            )

    def test_brokenpics(self):
        # create zero-size img in source_dir
        self.create_file(os.path.join(self.source_dir, "img.jpg"))

        dest_dir = self.get_working_path()
        self.album.generate(dest_dir)


class TestSpecialGens(LazygalTestGen):

    def setUp(self):
        super().setUp(False)
        self.dest_path = os.path.join(self.tmpdir, "dst")

    def test_paginate(self):
        """
        It shall be possible to split big galleries on mutiple index pages.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "thumbs-per-page", 4)
        self.setup_album(config)

        pics = ["img%d.jpg" % i for i in range(0, 9)]
        source_subgal = self.setup_subgal("subgal", pics)

        self.album.generate(self.dest_path)
        # FIXME: Check dest dir contents, test only catches uncaught exceptions
        # for now...

    def test_flatten(self):
        config = lazygal.config.LazygalConfig()
        config.set("global", "dir-flattening-depth", 0)
        self.setup_album(config)

        source_subgal = self.setup_subgal("subgal", ["subgal_img.jpg"])
        self.album.generate(self.dest_path)
        # FIXME: Check dest dir contents, test only catches uncaught exceptions
        # for now...

    def test_flattenpaginate(self):
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "thumbs-per-page", 4)
        config.set("global", "dir-flattening-depth", 0)
        self.setup_album(config)

        pics = ["img%d.jpg" % i for i in range(0, 9)]
        source_subgal = self.setup_subgal("subgal", pics)

        self.album.generate(self.dest_path)
        # FIXME: Check dest dir contents, test only catches uncaught exceptions
        # for now...

    @unittest.skipIf(not has_symlinks(), "symlinks not supported on platform")
    def test_dir_symlink(self):
        """
        The generator should follow symlinks on directories, but should not get
        stuck in infinite recursion if two distinct directory trees have
        symbolic links to each other.
        """
        self.setup_album()

        pics = ["img%d.jpg" % i for i in range(0, 2)]
        source_subgal = self.setup_subgal("subgal", pics)

        # cp -ar to create an out-of-tree source dir with pics
        src2_path = os.path.join(self.get_working_path(), "symlink_target")
        shutil.copytree(self.source_dir, src2_path)

        # symlink src2 so it show in album
        os.symlink(src2_path, os.path.join(self.source_dir, "symlinked"))
        # symlink src in src2 to check if the generator goes in an infinite
        # loop.
        os.symlink(self.source_dir, os.path.join(src2_path, "do_not_follow"))

        self.album.generate(self.dest_path)

        # Check root dir contents
        self.assertTrue(os.path.isdir(self.dest_path))
        for fn in ("index.html", "index_medium.html"):
            self.assertTrue(os.path.isfile(os.path.join(self.dest_path, fn)))
        for dn in ("subgal", "symlinked"):
            self.assertTrue(os.path.isdir(os.path.join(self.dest_path, dn)))

        # Check symlinked root contents
        dest_path = os.path.join(self.dest_path, "symlinked")
        for fn in ("index.html", "index_medium.html"):
            self.assertTrue(os.path.isfile(os.path.join(dest_path, fn)))

        # Check symlinked subgal contents
        dest_path = os.path.join(self.dest_path, "symlinked", "subgal")
        for fn in (
            "index.html",
            "index_medium.html",
            "img0.html",
            "img0_medium.html",
            "img0_thumb.jpg",
            "img0_small.jpg",
            "img0_medium.jpg",
            "img1.html",
            "img1_medium.html",
            "img1_thumb.jpg",
            "img1_small.jpg",
            "img1_medium.jpg",
        ):
            fp = os.path.join(dest_path, fn)
            self.assertTrue(os.path.isfile(fp), "%s is missing" % fp)

        # Check that symlinked initial root has not been processed
        self.assertFalse(
            os.path.isdir(
                os.path.join(self.dest_path, "symlinked", "subgal", "do_not_follow")
            )
        )


class TestSorting(LazygalTestGen):

    def setUp(self):
        super().setUp(False)
        self.dest_path = os.path.join(self.tmpdir, "dst")

    def __setup_pics(self, subgal_name=None):
        if subgal_name is None:
            subgal_name = "subgal"
        subgal_path = os.path.join(self.source_dir, subgal_name)
        os.mkdir(subgal_path)

        pics = [
            "4-december.jpg",
            "6-january.jpg",
            "1-february.jpg",
            "3-june.jpg",
            "5-august.jpg",
        ]
        months = [12, 1, 2, 6, 8]
        for index, pic in enumerate(pics):

            img_path = self.add_img(subgal_path, pic)
            img_exif = GExiv2.Metadata(img_path)
            for tag in (
                "Exif.Photo.DateTimeDigitized",
                "Exif.Photo.DateTimeOriginal",
                "Exif.Image.DateTime",
            ):
                img_exif[tag] = datetime.datetime(2011, months[index], 1).strftime(
                    "%Y:%m:%d %H:%M:%S"
                )
            img_exif.save_file()

        return subgal_path, pics

    def test_sortbyexif(self):
        """
        It shall be possible to sort images in a gallery according to EXIF
        date.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "sort-medias", "exif")
        self.setup_album(config)
        subgal_path, pics = self.__setup_pics()

        src_dir = Directory(subgal_path, [], pics, self.album)
        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)

        dest_subgal.call_populate_deps()
        dest_subgal.sort_task.make()

        self.assertEqual(
            [media.media.filename for media in dest_subgal.medias],
            [
                "6-january.jpg",
                "1-february.jpg",
                "3-june.jpg",
                "5-august.jpg",
                "4-december.jpg",
            ],
        )

    def test_sortbyexif_galsplit(self):
        """
        It shall be possible to sort images in galleries split on multiple
        pages according to EXIF date.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "sort-medias", "exif")
        config.set("webgal", "thumbs-per-page", 3)
        self.setup_album(config)
        subgal_path, pics = self.__setup_pics()

        src_dir = Directory(subgal_path, [], pics, self.album)
        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)

        dest_subgal.call_populate_deps()
        dest_subgal.sort_task.make()

        self.assertEqual(
            [media.media.filename for media in dest_subgal.medias],
            [
                "6-january.jpg",
                "1-february.jpg",
                "3-june.jpg",
                "5-august.jpg",
                "4-december.jpg",
            ],
        )

        # page #1
        page_medias = dest_subgal.index_pages[0][0].galleries[0][1]
        self.assertEqual(
            [media.media.filename for media in page_medias],
            ["6-january.jpg", "1-february.jpg", "3-june.jpg"],
        )
        # page #2
        page_medias = dest_subgal.index_pages[1][0].galleries[0][1]
        self.assertEqual(
            [media.media.filename for media in page_medias],
            ["5-august.jpg", "4-december.jpg"],
        )

    def test_sortbyfilename(self):
        """
        It shall be possible to sort images in a gallery by filename.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "sort-medias", "filename")
        self.setup_album(config)
        subgal_path, pics = self.__setup_pics()

        src_dir = Directory(subgal_path, [], pics, self.album)
        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)

        dest_subgal.call_populate_deps()
        dest_subgal.sort_task.make()

        self.assertEqual(
            [media.media.filename for media in dest_subgal.medias],
            [
                "1-february.jpg",
                "3-june.jpg",
                "4-december.jpg",
                "5-august.jpg",
                "6-january.jpg",
            ],
        )

    def test_sortbyfilename_galsplit(self):
        """
        It shall be possible to sort images in galleries split on multiple
        pages by filename.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "sort-medias", "filename")
        config.set("webgal", "thumbs-per-page", 3)
        self.setup_album(config)
        subgal_path, pics = self.__setup_pics()

        src_dir = Directory(subgal_path, [], pics, self.album)
        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)

        dest_subgal.call_populate_deps()
        dest_subgal.sort_task.make()

        self.assertEqual(
            [media.media.filename for media in dest_subgal.medias],
            [
                "1-february.jpg",
                "3-june.jpg",
                "4-december.jpg",
                "5-august.jpg",
                "6-january.jpg",
            ],
        )

        # page #1
        page_medias = dest_subgal.index_pages[0][0].galleries[0][1]
        self.assertEqual(
            [media.media.filename for media in page_medias],
            ["1-february.jpg", "3-june.jpg", "4-december.jpg"],
        )
        # page #2
        page_medias = dest_subgal.index_pages[1][0].galleries[0][1]
        self.assertEqual(
            [media.media.filename for media in page_medias],
            ["5-august.jpg", "6-january.jpg"],
        )

    def test_sortsubgals_dirnamereverse(self):
        """
        It shall be possible to sort sub-galleries accoring to the directory
        name.
        """
        config = lazygal.config.LazygalConfig()
        config.set("webgal", "sort-subgals", "dirname:reverse")
        self.setup_album(config)

        subgal_names = (
            "john",
            "2012_Trip",
            "albert",
            "1999_Christmas",
            "joe",
        )
        subgals_src = []
        subgals_dst = []
        for subgal_name in subgal_names:
            path, pics = self.__setup_pics(subgal_name)
            src = Directory(path, [], pics, self.album)
            dst = WebalbumDir(
                src, [], self.album, os.path.join(self.dest_path, subgal_name)
            )
            subgals_src.append(src)
            subgals_dst.append(dst)

        src_dir = Directory(self.source_dir, subgals_src, [], self.album)
        dest_subgal = WebalbumDir(src_dir, subgals_dst, self.album, self.dest_path)

        dest_subgal.call_populate_deps()
        dest_subgal.sort_task.make()

        self.assertEqual(
            [subgal.source_dir.name for subgal in dest_subgal.subgals],
            ["john", "joe", "albert", "2012_Trip", "1999_Christmas"],
        )


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2014 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
from . import LazygalTest
import lazygal.theme as t


class TestTheme(LazygalTest):

    def setUp(self):
        super().setUp()

        self.themes_dir = self.get_working_path()
        self.theme_name = "test_theme"
        self.theme_dir = os.path.join(self.themes_dir, self.theme_name)
        self.theme_manifest = os.path.join(self.theme_dir, "manifest.json")
        os.makedirs(self.theme_dir)

    def test_sharedfiles_prefixed(self):
        """
        File prefixed with SHARED_ in theme dir are copied in shared files
        directory.
        """
        prefixed = os.path.join(self.theme_dir, "SHARED_prefixed.txt")
        self.create_file(prefixed)
        theme = t.Theme(self.themes_dir, self.theme_name)

        self.assertEqual(len(theme.shared_files), 1)
        self.assertEqual(theme.shared_files[0]["source"], prefixed)
        self.assertEqual(theme.shared_files[0]["dest"], "prefixed.txt")

    def test_shared_file_manifest(self):
        """
        The manifest makes it possible to include files from other directories
        in shared files.
        """
        prefixed = os.path.join(self.theme_dir, "SHARED_prefixed.txt")
        self.create_file(prefixed)
        jslib = os.path.join(self.themes_dir, "lib-2.1.js")
        self.create_file(jslib)
        self.create_file(
            self.theme_manifest,
            """
{
    "shared": [
        {
            "path": "../lib-2.1.js",
            "dest": "lib.js"
        },
        {
            "path": "../lib-2.1.js",
            "dest": "js/"
        }
    ]
}
""",
        )

        theme = t.Theme(self.themes_dir, self.theme_name)

        shared_files = [(s["source"], s["dest"]) for s in theme.shared_files]
        self.assertEqual(
            shared_files,
            [(jslib, "lib.js"), (jslib, "js/lib-2.1.js"), (prefixed, "prefixed.txt")],
        )


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import sys
import os


__all__ = [
    "generators",
]


# Compute installation prefix
if os.path.isfile(os.path.join(os.path.dirname(__file__), "..", "setup.py")):
    INSTALL_MODE = "source"
    INSTALL_PREFIX = ""
else:
    # Lazygal is installed, assume we are in
    # $prefix/lib/pythonX/dist-packages/lazygal
    INSTALL_MODE = "installed"
    INSTALL_PREFIX = os.path.join(os.path.dirname(__file__), "..", "..", "..", "..")
    INSTALL_PREFIX = os.path.normpath(INSTALL_PREFIX)


def query_git_for_tag(gitdir):
    import subprocess

    o = (
        subprocess.check_output(
            (
                "git",
                "-C",
                os.path.dirname(__file__),
                "describe",
                "--tags",
            )
        )
        .decode(sys.stdout.encoding)
        .strip()
    )
    tokens = o.split("-")
    if len(tokens[-1]) != 8:
        # this is not a hash
        return None
    elif tokens[0].startswith("debian/"):
        return None
    else:
        return "%s+%s.%s" % tuple(tokens)


def git_repo_mtime(gitdir):
    return max(
        [
            os.path.getmtime(os.path.join(gitdir, "index")),
            os.path.getmtime(os.path.join(gitdir, "HEAD")),
            os.path.getmtime(os.path.join(gitdir, "refs", "tags")),
        ]
    )


def get_git_rev():
    gitdir = os.path.join(os.path.dirname(__file__), "..", ".git")
    if os.path.isdir(gitdir):
        last_revision_cache = os.path.join(gitdir, "last_revision")

        if os.path.isfile(last_revision_cache) and os.path.getmtime(
            last_revision_cache
        ) > git_repo_mtime(gitdir):
            with open(last_revision_cache, "r") as fp:
                return fp.read()
        else:
            import subprocess

            try:
                lastrev = query_git_for_tag(gitdir)
            except subprocess.CalledProcessError:
                if os.path.isfile(last_revision_cache):
                    os.unlink(last_revision_cache)
                return ""
            else:
                if lastrev:
                    with open(last_revision_cache, "w") as fp:
                        fp.write(lastrev)
                    return lastrev
                else:
                    return ""
    else:
        return ""


__version__ = "0.11"

rev = get_git_rev()
if rev:
    __version__ = rev

# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime
import os
import time

from genshi.core import START
from genshi.template import TemplateLoader, MarkupTemplate, NewTextTemplate
from genshi.template import TemplateNotFound
from genshi.template.eval import UndefinedError
from genshi.input import XMLParser

import lazygal


class LazygalTemplate(object):

    def __init__(self, loader, genshi_tpl):
        self.loader = loader
        self.path = genshi_tpl.filepath
        self.genshi_tpl = genshi_tpl

    def __complement_values(self, values):
        values["gen_datetime"] = datetime.datetime.now()

        # using time.strftime() here instead of datetime.strftime() because
        # the latter does not carry the current timezone info, and '%c' in some
        # locales needs to display tzname.
        values["gen_date"] = time.strftime("%c")

        values["lazygal_version"] = lazygal.__version__

        return values

    def __generate(self, values):
        # The cryptic values['_'] = _ is the way to pass the gettext
        # translation function to the templates : the _() callable is assigned
        # to the '_' keyword arg.
        values["_"] = _
        return self.genshi_tpl.generate(**values)

    def instanciate(self, values):
        self.__complement_values(values)
        # encoding=None gives us a unicode string instead of an utf-8 encoded
        # string. This is because we are not out of lazygal yet.
        return self.__generate(values).render(self.serialization_method, encoding=None)

    def dump(self, values, dest):
        self.__complement_values(values)

        page = open(dest, "wb")
        try:
            self.__generate(values).render(
                method=self.serialization_method, out=page, encoding="utf-8"
            )
        except UndefinedError as e:
            print("W: %s" % e)
            raise
        finally:
            page.close()


class XmlTemplate(LazygalTemplate):

    serialization_method = "xhtml"
    genshi_tpl_class = MarkupTemplate

    def subtemplates(self, tpl=None):
        if tpl is None:
            tpl = self

        subtemplates = []
        f = open(tpl.path, "r")
        try:
            for kind, data, pos in XMLParser(f, filename=tpl.path):
                if kind is START:
                    tag, attrib = data
                    if (
                        tag.namespace == "http://www.w3.org/2001/XInclude"
                        and tag.localname == "include"
                    ):
                        subtpl_ident = attrib.get("href")
                        try:
                            subtpl = self.loader.load(subtpl_ident)
                        except TemplateNotFound:
                            # This will fail later, here we just need to ignore
                            # template idents that are dynamically computed.
                            pass
                        else:
                            subtemplates.append(subtpl)
        finally:
            f.close()

        for subtemplate in subtemplates:
            for new_subtpl in self.subtemplates(subtemplate):
                if new_subtpl not in subtemplates:
                    subtemplates.append(new_subtpl)

        return subtemplates


class PlainTemplate(LazygalTemplate):

    serialization_method = "text"
    genshi_tpl_class = NewTextTemplate


class TplFactory(object):

    known_exts = {
        ".thtml": XmlTemplate,
        ".tcss": PlainTemplate,
        ".tjs": PlainTemplate,
    }

    def __init__(self, default_tpl_dir, tpl_dir):
        # We use lenient mode here because we want an easy way to check whether
        # a template variable is defined, or the empty string, thus defined()
        # will only work for the 'whether it is defined' part of the test.
        self.loader = TemplateLoader(
            [tpl_dir, default_tpl_dir], variable_lookup="lenient"
        )

    def is_known_template_type(self, file):
        filename, ext = os.path.splitext(os.path.basename(file))
        return ext in self.known_exts.keys()

    def load(self, tpl_ident):
        if self.is_known_template_type(tpl_ident):
            filename, ext = os.path.splitext(os.path.basename(tpl_ident))
            tpl_class = self.known_exts[ext]
            tpl = self.loader.load(tpl_ident, cls=tpl_class.genshi_tpl_class)
            return tpl_class(self, tpl)
        else:
            raise ValueError(_("Unknown template type for %s" % tpl_ident))


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2010-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import shutil
import tempfile
import unittest
import codecs

from lazygal.sourcetree import Directory
from lazygal.generators import Album


SAMPLES_DIR = os.path.dirname(__file__)
SAMPLE_IMG = os.path.join(SAMPLES_DIR, "sample.jpg")
SAMPLE_VIDEO_MOV = os.path.join(SAMPLES_DIR, "vid.mov")


# Init i18n
import gettext

LOCALES_PATH = os.path.normpath(os.path.join(os.path.dirname(__file__), "build", "mo"))
gettext.install("lazygal", LOCALES_PATH)


# Init logging
import logging

logging.basicConfig(format="%(message)s", level=logging.ERROR)
logger = logging.getLogger()
logger.setLevel(logging.INFO)
try:
    debug = os.environ["DEBUG"]
except KeyError:
    logger.setLevel(logging.ERROR)
else:
    logger.setLevel(logging.DEBUG)


def has_symlinks():
    try:
        os.symlink
    except AttributeError:
        return False
    else:
        return True


class LazygalTest(unittest.TestCase):

    def setUp(self):
        self.__workdirs = []

    def get_working_path(self):
        new_wd = tempfile.mkdtemp()
        self.__workdirs.append(new_wd)
        return new_wd

    def get_sample_path(self, sample):
        return os.path.join(SAMPLES_DIR, sample)

    def add_img(self, dest_dir, name):
        img_path = os.path.join(dest_dir, name)
        shutil.copy(SAMPLE_IMG, img_path)
        return img_path

    def add_video(self, dest_dir, name):
        vid_path = os.path.join(dest_dir, name)
        shutil.copy(SAMPLE_VIDEO_MOV, vid_path)
        return vid_path

    def create_file(self, path, contents=""):
        with codecs.open(path, "w", "utf-8") as f:
            f.write(contents)

    def tearDown(self):
        for wd in self.__workdirs:
            shutil.rmtree(wd)


class LazygalTestGen(LazygalTest):

    def setUp(self, album=True):
        super().setUp()

        self.tmpdir = self.get_working_path()
        self.source_dir = os.path.join(self.tmpdir, "src")
        os.mkdir(self.source_dir)

        if album:
            self.setup_album()

    def setup_album(self, config=None):
        self.album = Album(self.source_dir, config)

    def setup_subgal(self, name, pic_names, vid_names=None):
        subgal_path = os.path.join(self.source_dir, name)
        if not os.path.isdir(subgal_path):
            os.mkdir(subgal_path)
        for pic_name in pic_names:
            self.add_img(subgal_path, pic_name)
        if vid_names:
            for vid_name in vid_names:
                self.add_video(subgal_path, vid_name)

        return Directory(subgal_path, [], pic_names, self.album)


# Workaround for __import__ behavior, see
# http://docs.python.org/lib/built-in-funcs.html
def my_import(name):
    mod = __import__(name)
    components = name.split(".")
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


def sample_album(outdir):
    d = os.path.normpath(outdir)
    os.makedirs(d, exist_ok=True)
    sample = LazygalTest()
    sample.add_img(d, "root.jpg")

    flowers = os.path.join(d, "flower")
    os.makedirs(flowers, exist_ok=True)
    sample.add_img(flowers, "coquelicot.jpg")
    sample.add_img(flowers, "rose.jpg")
    sample.add_video(flowers, "nature.mov")

    beers = os.path.join(d, "beers")
    os.makedirs(beers, exist_ok=True)
    sample.add_img(beers, "guinness.jpg")
    sample.add_img(beers, "kilkenny.jpg")


def run():
    import glob

    suitelist = []
    for fn in glob.glob(os.path.join(os.path.dirname(__file__), "test_*.py")):
        module_path = ".".join(["lazygaltest", os.path.basename(fn[:-3])])
        m = my_import(module_path)
        suitelist.append(unittest.defaultTestLoader.loadTestsFromModule(m))
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(unittest.TestSuite(suitelist))


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
from datetime import datetime


from gi.repository import GExiv2


from . import LazygalTest
from lazygal.generators import Album
from lazygal.sourcetree import Directory


class TestSourceTree(LazygalTest):

    def setUp(self):
        super().setUp()

        self.source_dir = self.get_working_path()
        self.album = Album(self.source_dir)

    def get_dir(self, drpath):
        """
        Returns a directory object inside the test album root.
        """
        dpath = os.path.join(self.source_dir, drpath)
        os.makedirs(dpath)
        return Directory(dpath, [], [], self.album)

    def test_skipped(self):
        d = self.get_dir("joe/young")
        self.assertEqual(d.should_be_skipped(), False, d.path)

        d = self.get_dir(".svn/young")
        self.assertEqual(d.should_be_skipped(), True, d.path)

        d = self.get_dir("joe/young/.git")
        self.assertEqual(d.should_be_skipped(), True, d.path)

    def test_dir_parent_paths(self):
        d = self.get_dir("joe/young/early_years")

        expected = [
            os.path.join(self.source_dir, "joe/young/early_years"),
            os.path.join(self.source_dir, "joe/young"),
            os.path.join(self.source_dir, "joe"),
            self.source_dir,
        ]
        self.assertEqual(d.parent_paths(), expected)

    def test_latest_media_stamp(self):
        dpath = os.path.join(self.source_dir, "srcdir")
        os.makedirs(dpath)

        pics = ["pic1.jpg", "pic2.jpg", "pic3.jpg"]

        for fn in pics:
            self.add_img(dpath, fn)

        # no exif test
        imgpath = os.path.join(dpath, "pic1.jpg")
        img = GExiv2.Metadata(imgpath)
        del img["Exif.Photo.DateTimeDigitized"]
        del img["Exif.Photo.DateTimeOriginal"]
        img.save_file()
        os.utime(imgpath, (0, datetime(2011, 7, 3).timestamp()))
        imgpath = os.path.join(dpath, "pic2.jpg")
        img = GExiv2.Metadata(imgpath)
        del img["Exif.Photo.DateTimeDigitized"]
        del img["Exif.Photo.DateTimeOriginal"]
        img.save_file()
        os.utime(imgpath, (0, datetime(2011, 7, 4).timestamp()))
        imgpath = os.path.join(dpath, "pic3.jpg")
        img = GExiv2.Metadata(imgpath)
        del img["Exif.Photo.DateTimeDigitized"]
        del img["Exif.Photo.DateTimeOriginal"]
        img.save_file()
        os.utime(imgpath, (0, datetime(2011, 7, 2).timestamp()))
        d = Directory(dpath, [], pics, self.album)
        self.assertEqual(
            d.latest_media_stamp(from_media=True), datetime(2011, 7, 4).timestamp()
        )

        # mixed exif and no exif test
        imgpath = os.path.join(dpath, "pic2.jpg")
        img = GExiv2.Metadata(imgpath)
        img["Exif.Photo.DateTimeOriginal"] = datetime(2015, 7, 4).strftime(
            "%Y:%m:%d %H:%M:%S"
        )
        img.save_file()
        d = Directory(dpath, [], pics, self.album)
        self.assertEqual(
            d.latest_media_stamp(from_media=True), datetime(2015, 7, 4).timestamp()
        )

        # full exif
        imgpath = os.path.join(dpath, "pic1.jpg")
        img = GExiv2.Metadata(imgpath)
        img["Exif.Photo.DateTimeOriginal"] = datetime(2015, 8, 1).strftime(
            "%Y:%m:%d %H:%M:%S"
        )
        img.save_file()
        imgpath = os.path.join(dpath, "pic3.jpg")
        img = GExiv2.Metadata(imgpath)
        img["Exif.Photo.DateTimeOriginal"] = datetime(2015, 8, 20).strftime(
            "%Y:%m:%d %H:%M:%S"
        )
        img.save_file()
        d = Directory(dpath, [], pics, self.album)
        self.assertEqual(
            d.latest_media_stamp(from_media=True), datetime(2015, 8, 20).timestamp()
        )


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab
